/* A Bison parser, made by GNU Bison 3.7.4.  */

/* Bison implementation for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
   especially those whose name start with YY_ or yy_.  They are
   private implementation details that can be changed or removed.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Identify Bison output, and Bison version.  */
#define YYBISON 30704

/* Bison version string.  */
#define YYBISON_VERSION "3.7.4"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 0

/* Push parsers.  */
#define YYPUSH 0

/* Pull parsers.  */
#define YYPULL 1




/* First part of user prologue.  */
#line 17 "gram.y"

/*
 *  pmlogrewrite parser
 */
#include "pmapi.h"
#include "libpcp.h"
#include "logger.h"
#include <errno.h>
#include <assert.h>

#define PM_TEXT_TYPE_MASK (PM_TEXT_ONELINE | PM_TEXT_HELP)

static indomspec_t	*current_indomspec;
static int		current_star_indom;
static int		do_walk_indom;
static int		star_domain;

static metricspec_t	*current_metricspec;
static int		current_star_metric;
static int		star_cluster;
static int		do_walk_metric;
static int		output = OUTPUT_ALL;
static int		one_inst;
static char		*one_name;

static textspec_t	*current_textspec;
static int		do_walk_text;

static int		current_label_id;
static char *		current_label_name;
static char *		current_label_value;
static int		current_label_instance;
static char *		current_label_instance_name;
static labelspec_t	*current_labelspec;
static int		do_walk_label;

indomspec_t *
walk_indom(int mode)
{
    static indomspec_t	*ip;

    if (do_walk_indom) {
	if (mode == W_START)
	    ip = indom_root;
	else
	    ip = ip->i_next;
	while (ip != NULL && pmInDom_domain(ip->old_indom) != star_domain)
	    ip = ip->i_next;
    }
    else {
	if (mode == W_START)
	    ip = current_indomspec;
	else
	    ip = NULL;
    }

    return ip;
}

metricspec_t *
walk_metric(int mode, int flag, char *which, int dupok)
{
    static metricspec_t	*mp;

    if (do_walk_metric) {
	if (mode == W_START)
	    mp = metric_root;
	else
	    mp = mp->m_next;
	while (mp != NULL) {
	    if (pmID_domain(mp->old_desc.pmid) == star_domain &&
		(star_cluster == PM_ID_NULL || star_cluster == pmID_cluster(mp->old_desc.pmid)))
		break;
	    mp = mp->m_next;
	}
    }
    else {
	if (mode == W_START)
	    mp = current_metricspec;
	else
	    mp = NULL;
    }

    if (mp != NULL) {
	if (!dupok && (mp->flags & flag)) {
	    pmsprintf(mess, sizeof(mess), "Duplicate %s clause for metric %s", which, mp->old_name);
	    yyerror(mess);
	}
	if (flag != METRIC_DELETE) {
	    if (mp->flags & METRIC_DELETE) {
		pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted metric %s", which, mp->old_name);
		yyerror(mess);
	    }
	}
	else {
	    if (mp->flags & (~METRIC_DELETE)) {
		pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for metric %s", mp->old_name);
		yyerror(mess);
	    }
	}
    }

    return mp;
}

static const char *
textTypeStr(int type)
{
    switch(type)
	{
	case PM_TEXT_ONELINE:
	    return "one line";
	case PM_TEXT_HELP:
	    return "help";
	default:
	    break;
	}

    return "unknown";
}
 
textspec_t *
walk_text(int mode, int flag, char *which, int dupok)
{
    static textspec_t	*tp;

    if (do_walk_text) {
	if (mode == W_START)
	    tp = text_root;
	else
	    tp = tp->t_next;

	/* Only consider the active specs. */
	while (tp != NULL && (tp->flags & TEXT_ACTIVE) == 0)
	    tp = tp->t_next;
    }
    else {
	if (mode == W_START) {
	    tp = current_textspec;
	    if (tp)
		assert ((tp->flags & TEXT_ACTIVE));
	}
	else
	    tp = NULL;
    }

    if (tp != NULL) {
	tp->flags &= ~TEXT_ACTIVE;
	
	if (!dupok && (tp->flags & flag)) {
	    if ((tp->old_type & PM_TEXT_PMID)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for %s text for metric %s",
			  which, textTypeStr(tp->old_type), pmIDStr(tp->old_id));
	    }
	    else {
		assert((tp->old_type & PM_TEXT_INDOM));
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for %s text for indom %s",
			  which, textTypeStr(tp->old_type), pmInDomStr(tp->old_id));
	    }
	    yyerror(mess);
	}
	if (flag != TEXT_DELETE) {
	    if (tp->flags & TEXT_DELETE) {
		if ((tp->old_type & PM_TEXT_PMID)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted %s text for metric %s",
			      which, textTypeStr(tp->old_type), pmIDStr(tp->old_id));
		}
		else {
		    assert((tp->old_type & PM_TEXT_INDOM));
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted %s text for indom %s",
			      which, textTypeStr(tp->old_type), pmInDomStr(tp->old_id));
		}
		yyerror(mess);
	    }
	}
	else {
	    if (tp->flags & (~TEXT_DELETE)) {
		if ((tp->old_type & PM_TEXT_PMID)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for %s text for metric %s",
			      textTypeStr(tp->old_type), pmIDStr(tp->old_id));
		}
		else {
		    assert((tp->old_type & PM_TEXT_INDOM));
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for %s text for indom %s",
			      textTypeStr(tp->old_type), pmInDomStr(tp->old_id));
		}
		yyerror(mess);
	    }
	}
    }

    return tp;
}

labelspec_t *
walk_label(int mode, int flag, char *which, int dupok)
{
    static labelspec_t	*lp;

    if (do_walk_label > 1) {
	if (mode == W_START)
	    lp = label_root;
	else
	    lp = lp->l_next;

	/* Only consider the active specs. */
	while (lp != NULL && (lp->flags & LABEL_ACTIVE) == 0)
	    lp = lp->l_next;
    }
    else {
	if (mode == W_START) {
	    lp = current_labelspec;
	    if (lp)
		assert ((lp->flags & LABEL_ACTIVE));
	}
	else
	    lp = NULL;
    }

    if (lp != NULL) {
	lp->flags &= ~LABEL_ACTIVE;
	
	if (!dupok && (lp->flags & flag)) {
	    if ((lp->old_type & PM_LABEL_CONTEXT)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for context label",
			  which);
	    }
	    else if ((lp->old_type & PM_LABEL_DOMAIN)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for domain %d",
			  which, pmID_domain(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_CLUSTER)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for cluster %d.%d",
			  which, pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_ITEM)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for metric %s",
			  which, pmIDStr(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_INDOM)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for indom %s",
			  which, pmInDomStr(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_INSTANCES)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for the instances of indom %s",
			  which, pmInDomStr(lp->old_id));
	    }
	    yyerror(mess);
	}
	if (flag != LABEL_DELETE) {
	    if (lp->flags & LABEL_DELETE) {
		if ((lp->old_type & PM_LABEL_CONTEXT)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted context label",
			      which);
		}
		else if ((lp->old_type & PM_LABEL_DOMAIN)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for domain %d",
			      which, pmID_domain(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_CLUSTER)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for cluster %d.%d",
			      which, pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_ITEM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for metric %s",
			      which, pmIDStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INDOM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for indom %s",
			      which, pmInDomStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INSTANCES)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for the instances of indom %s",
			      which, pmInDomStr(lp->old_id));
		}
		yyerror(mess);
	    }
	}
	else {
	    if (lp->flags & (~LABEL_DELETE)) {
		if ((lp->old_type & PM_LABEL_CONTEXT)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for context label");
		}
		else if ((lp->old_type & PM_LABEL_DOMAIN)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for domain %d",
			      pmID_domain(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_CLUSTER)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for cluster %d.%d",
			      pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_ITEM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for metric %s",
			      pmIDStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INDOM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for indom %s",
			      pmInDomStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INSTANCES)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for the instances of indom %s",
			      pmInDomStr(lp->old_id));
		}
		yyerror(mess);
	    }
	}
    }

    return lp;
}

void
deactivate_labels(void)
{
    labelspec_t	*lp;
    for (lp = walk_label(W_START, LABEL_ACTIVE, "active", 0); lp != NULL; lp = walk_label(W_NEXT, 0, "", 0)) {
	lp->flags &= ~LABEL_ACTIVE;
    }
}

void
new_context_label()
{
    labelspec_t	*lp;
    int		sts;
    char	buf[PM_MAXLABELJSONLEN];

    /*
     * Ignore the other change specs which were identified and
     * add the new label to the generic context label change spec.
     * Search for an existing one first.
     */
    deactivate_labels();
    for (lp = label_root; lp != NULL; lp = lp->l_next) {
	if (lp->old_type == PM_LABEL_CONTEXT) {
	    assert (lp->old_id == PM_ID_NULL);
	    if (lp->old_label == NULL)
		break;
	}
    }

    /* Create one if none exists. */
    if (lp == NULL)
	lp = create_label(PM_LABEL_CONTEXT, PM_ID_NULL, 0, NULL, NULL);

    /* Add the new label to the label change spec. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);
    if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_CONTEXT)) < 0) {
	pmsprintf(mess, sizeof(mess),
		  "Unable to add new context label %s: %s",
		  buf, pmErrStr(sts));
	yyerror(mess);
    }
    lp->new_labels->inst = 0;
    lp->flags |= LABEL_NEW;
}

void
new_domain_label(int domain)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN];

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashpmid;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmID_domain(node->key) != domain)
	    continue;

	/*
	 * A metric in this domain exists.
	 * Add the new label to the generic label change spec for this
	 * domain. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_DOMAIN)
		continue;
	    if (lp->old_id != domain)
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	/* Create one if none exists. */
	if (lp == NULL)
	    lp = create_label(PM_LABEL_DOMAIN, domain, 0, NULL, NULL);

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_DOMAIN)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new domain label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* We only need to find one metric in the specified domain. */
	break;
    }

    /* Did we find any clusters matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching domain for new label %s", buf);
	yywarn(mess);
    }
}

void
new_cluster_label(int cluster)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashpmid;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmID_domain(node->key) != pmID_domain(cluster))
	    continue;
	if (! current_star_metric && 
	    pmID_cluster(node->key) != pmID_cluster(cluster))
	    continue;

	/*
	 * A metric in this cluster exists.
	 * Add the new label to the generic label change spec for the
	 * identified cluster. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_CLUSTER)
		continue;
	    if (pmID_domain(lp->old_id) != pmID_domain((pmID)(node->key)))
		continue;
	    if (pmID_cluster(lp->old_id) != pmID_cluster((pmID)(node->key)))
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	if (lp == NULL) {
	    pmID current_cluster =
		pmID_build(pmID_domain(node->key), pmID_cluster(node->key), 0);
	    lp = create_label(PM_LABEL_CLUSTER, current_cluster, 0, NULL, NULL);
	}

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_CLUSTER)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new cluster label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* Do we need to look for more clusters? */
	if (! current_star_metric)
	    break;
    }

    /* Did we find any clusters matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching cluster for new label %s", buf);
	yywarn(mess);
    }
}

void
new_item_label(int item)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashpmid;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmID_domain(node->key) != pmID_domain(item))
	    continue;
	if (current_star_metric < 2) {
	    if (pmID_cluster(node->key) != pmID_cluster(item))
		continue;
	    if (! current_star_metric && (pmID)(node->key) != (pmID)item)
		continue;
	}

	/*
	 * A metric with this pmID exists.
	 * Add the new label to the generic label change spec for the
	 * identified item. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_ITEM)
		continue;
	    if ((pmID)(lp->old_id) != (pmID)(node->key))
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	if (lp == NULL)
	    lp = create_label(PM_LABEL_ITEM, (pmID)node->key, 0, NULL, NULL);

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_ITEM)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new metric item label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* Do we need to look for more items? */
	if (! current_star_metric)
	    break;
    }

    /* Did we find any items matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching metric for new label %s", buf);
	yywarn(mess);
    }
}

void
new_indom_label(int indom)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashindom;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmInDom_domain(node->key) != pmInDom_domain(indom))
	    continue;
	if (! current_star_indom && 
	    pmInDom_serial(node->key) != pmInDom_serial(indom))
	    continue;

	/*
	 * A metric in this indom exists.
	 * Add the new label to the generic label change spec for the
	 * identified indom. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_INDOM)
		continue;
	    if (pmInDom_domain(lp->old_id) != pmInDom_domain((pmInDom)(node->key)))
		continue;
	    if (pmInDom_serial(lp->old_id) != pmInDom_serial((pmInDom)(node->key)))
		continue;
	    if (lp->old_instance != -1)
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	if (lp == NULL) {
	    pmInDom current_indom =
		pmInDom_build(pmInDom_domain(node->key), pmInDom_serial(node->key));
	    lp = create_label(PM_LABEL_INDOM, current_indom, 0, NULL, NULL);
	}

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_INDOM)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new indom label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* Do we need to look for more indoms? */
	if (! current_star_indom)
	    break;
    }

    /* Did we find any indoms matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching indom for new label %s", buf);
	yywarn(mess);
    }
}

void
new_indom_instance_label(int indom)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    __pmLogInDom	*idp;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashindom;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmInDom_domain(node->key) != pmInDom_domain(indom))
	    continue;
	if (! current_star_indom && 
	    pmInDom_serial(node->key) != pmInDom_serial(indom))
	    continue;

	/*
	 * A metric in this indom exists.
	 * Add the new label to the generic label change specs for the
	 * identified indom instances. Search for existing ones first.
	 */
	for (idp = (__pmLogInDom *)node->data; idp != NULL; idp = idp->next) {
	    int indom_ix;
	    for (indom_ix = 0; indom_ix < idp->numinst; ++indom_ix) {
		int instance = idp->instlist[indom_ix];
		if (current_label_instance != -1 && instance != current_label_instance)
		    continue;
		found = 1;
		for (lp = label_root; lp != NULL; lp = lp->l_next) {
		    if (lp->old_type != PM_LABEL_INSTANCES)
			continue;
		    if (pmInDom_domain(lp->old_id) != pmInDom_domain((pmInDom)(node->key)))
			continue;
		    if (pmInDom_serial(lp->old_id) != pmInDom_serial((pmInDom)(node->key)))
			continue;
		    if (lp->old_instance != instance)
			continue;
		    if (lp->old_label == NULL)
			break;
		}

		if (lp == NULL) {
		    pmInDom current_indom =
			pmInDom_build(pmInDom_domain(node->key), pmInDom_serial(node->key));
		    lp = create_label(PM_LABEL_INSTANCES, current_indom,
				      instance, NULL, NULL);
		}

		/* Add the new label to the label change spec. */
		if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_INSTANCES)) < 0) {
		    pmsprintf(mess, sizeof(mess),
			      "Unable to add new indom instance label %s: %s",
			      buf, pmErrStr(sts));
		    yyerror(mess);
		}
		lp->new_labels->inst = instance;
		lp->flags |= LABEL_NEW;
	    }
	}

	/* Do we need to look for more indoms? */
	if (! current_star_indom)
	    break;
    }

    /* Did we find any indom instances matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching indom instance for new label %s", buf);
	yywarn(mess);
    }
}


#line 880 "gram.tab.c"

# ifndef YY_CAST
#  ifdef __cplusplus
#   define YY_CAST(Type, Val) static_cast<Type> (Val)
#   define YY_REINTERPRET_CAST(Type, Val) reinterpret_cast<Type> (Val)
#  else
#   define YY_CAST(Type, Val) ((Type) (Val))
#   define YY_REINTERPRET_CAST(Type, Val) ((Type) (Val))
#  endif
# endif
# ifndef YY_NULLPTR
#  if defined __cplusplus
#   if 201103L <= __cplusplus
#    define YY_NULLPTR nullptr
#   else
#    define YY_NULLPTR 0
#   endif
#  else
#   define YY_NULLPTR ((void*)0)
#  endif
# endif

#include "gram.tab.h"
/* Symbol kind.  */
enum yysymbol_kind_t
{
  YYSYMBOL_YYEMPTY = -2,
  YYSYMBOL_YYEOF = 0,                      /* "end of file"  */
  YYSYMBOL_YYerror = 1,                    /* error  */
  YYSYMBOL_YYUNDEF = 2,                    /* "invalid token"  */
  YYSYMBOL_TOK_LBRACE = 3,                 /* TOK_LBRACE  */
  YYSYMBOL_TOK_RBRACE = 4,                 /* TOK_RBRACE  */
  YYSYMBOL_TOK_PLUS = 5,                   /* TOK_PLUS  */
  YYSYMBOL_TOK_MINUS = 6,                  /* TOK_MINUS  */
  YYSYMBOL_TOK_COLON = 7,                  /* TOK_COLON  */
  YYSYMBOL_TOK_COMMA = 8,                  /* TOK_COMMA  */
  YYSYMBOL_TOK_ASSIGN = 9,                 /* TOK_ASSIGN  */
  YYSYMBOL_TOK_GLOBAL = 10,                /* TOK_GLOBAL  */
  YYSYMBOL_TOK_INDOM = 11,                 /* TOK_INDOM  */
  YYSYMBOL_TOK_DUPLICATE = 12,             /* TOK_DUPLICATE  */
  YYSYMBOL_TOK_METRIC = 13,                /* TOK_METRIC  */
  YYSYMBOL_TOK_HOSTNAME = 14,              /* TOK_HOSTNAME  */
  YYSYMBOL_TOK_TZ = 15,                    /* TOK_TZ  */
  YYSYMBOL_TOK_TIME = 16,                  /* TOK_TIME  */
  YYSYMBOL_TOK_NAME = 17,                  /* TOK_NAME  */
  YYSYMBOL_TOK_INST = 18,                  /* TOK_INST  */
  YYSYMBOL_TOK_INAME = 19,                 /* TOK_INAME  */
  YYSYMBOL_TOK_DELETE = 20,                /* TOK_DELETE  */
  YYSYMBOL_TOK_PMID = 21,                  /* TOK_PMID  */
  YYSYMBOL_TOK_NULL_INT = 22,              /* TOK_NULL_INT  */
  YYSYMBOL_TOK_TYPE = 23,                  /* TOK_TYPE  */
  YYSYMBOL_TOK_IF = 24,                    /* TOK_IF  */
  YYSYMBOL_TOK_SEM = 25,                   /* TOK_SEM  */
  YYSYMBOL_TOK_UNITS = 26,                 /* TOK_UNITS  */
  YYSYMBOL_TOK_OUTPUT = 27,                /* TOK_OUTPUT  */
  YYSYMBOL_TOK_RESCALE = 28,               /* TOK_RESCALE  */
  YYSYMBOL_TOK_TEXT = 29,                  /* TOK_TEXT  */
  YYSYMBOL_TOK_ONELINE = 30,               /* TOK_ONELINE  */
  YYSYMBOL_TOK_HELP = 31,                  /* TOK_HELP  */
  YYSYMBOL_TOK_TEXT_STAR = 32,             /* TOK_TEXT_STAR  */
  YYSYMBOL_TOK_LABEL = 33,                 /* TOK_LABEL  */
  YYSYMBOL_TOK_CONTEXT = 34,               /* TOK_CONTEXT  */
  YYSYMBOL_TOK_LABEL_STAR = 35,            /* TOK_LABEL_STAR  */
  YYSYMBOL_TOK_DOMAIN = 36,                /* TOK_DOMAIN  */
  YYSYMBOL_TOK_CLUSTER = 37,               /* TOK_CLUSTER  */
  YYSYMBOL_TOK_ITEM = 38,                  /* TOK_ITEM  */
  YYSYMBOL_TOK_INSTANCES = 39,             /* TOK_INSTANCES  */
  YYSYMBOL_TOK_INSTANCE = 40,              /* TOK_INSTANCE  */
  YYSYMBOL_TOK_NEW = 41,                   /* TOK_NEW  */
  YYSYMBOL_TOK_VALUE = 42,                 /* TOK_VALUE  */
  YYSYMBOL_TOK_GNAME = 43,                 /* TOK_GNAME  */
  YYSYMBOL_TOK_NUMBER = 44,                /* TOK_NUMBER  */
  YYSYMBOL_TOK_STRING = 45,                /* TOK_STRING  */
  YYSYMBOL_TOK_TEXT_STRING = 46,           /* TOK_TEXT_STRING  */
  YYSYMBOL_TOK_HNAME = 47,                 /* TOK_HNAME  */
  YYSYMBOL_TOK_FLOAT = 48,                 /* TOK_FLOAT  */
  YYSYMBOL_TOK_JSON_STRING = 49,           /* TOK_JSON_STRING  */
  YYSYMBOL_TOK_JSON_NUMBER = 50,           /* TOK_JSON_NUMBER  */
  YYSYMBOL_TOK_JSON_TRUE = 51,             /* TOK_JSON_TRUE  */
  YYSYMBOL_TOK_JSON_FALSE = 52,            /* TOK_JSON_FALSE  */
  YYSYMBOL_TOK_JSON_NULL = 53,             /* TOK_JSON_NULL  */
  YYSYMBOL_TOK_INDOM_STAR = 54,            /* TOK_INDOM_STAR  */
  YYSYMBOL_TOK_PMID_INT = 55,              /* TOK_PMID_INT  */
  YYSYMBOL_TOK_PMID_STAR = 56,             /* TOK_PMID_STAR  */
  YYSYMBOL_TOK_TYPE_NAME = 57,             /* TOK_TYPE_NAME  */
  YYSYMBOL_TOK_SEM_NAME = 58,              /* TOK_SEM_NAME  */
  YYSYMBOL_TOK_SPACE_NAME = 59,            /* TOK_SPACE_NAME  */
  YYSYMBOL_TOK_TIME_NAME = 60,             /* TOK_TIME_NAME  */
  YYSYMBOL_TOK_COUNT_NAME = 61,            /* TOK_COUNT_NAME  */
  YYSYMBOL_TOK_OUTPUT_TYPE = 62,           /* TOK_OUTPUT_TYPE  */
  YYSYMBOL_YYACCEPT = 63,                  /* $accept  */
  YYSYMBOL_config = 64,                    /* config  */
  YYSYMBOL_speclist = 65,                  /* speclist  */
  YYSYMBOL_spec = 66,                      /* spec  */
  YYSYMBOL_globalspec = 67,                /* globalspec  */
  YYSYMBOL_globaloptlist = 68,             /* globaloptlist  */
  YYSYMBOL_globalopt = 69,                 /* globalopt  */
  YYSYMBOL_hname = 70,                     /* hname  */
  YYSYMBOL_signnumber = 71,                /* signnumber  */
  YYSYMBOL_number = 72,                    /* number  */
  YYSYMBOL_float = 73,                     /* float  */
  YYSYMBOL_signtime = 74,                  /* signtime  */
  YYSYMBOL_time = 75,                      /* time  */
  YYSYMBOL_indomspec = 76,                 /* indomspec  */
  YYSYMBOL_77_1 = 77,                      /* $@1  */
  YYSYMBOL_indom_int = 78,                 /* indom_int  */
  YYSYMBOL_optindomopt = 79,               /* optindomopt  */
  YYSYMBOL_indomoptlist = 80,              /* indomoptlist  */
  YYSYMBOL_indomopt = 81,                  /* indomopt  */
  YYSYMBOL_duplicateopt = 82,              /* duplicateopt  */
  YYSYMBOL_metricspec = 83,                /* metricspec  */
  YYSYMBOL_84_2 = 84,                      /* $@2  */
  YYSYMBOL_pmid_or_name = 85,              /* pmid_or_name  */
  YYSYMBOL_pmid_int = 86,                  /* pmid_int  */
  YYSYMBOL_optmetricoptlist = 87,          /* optmetricoptlist  */
  YYSYMBOL_metricoptlist = 88,             /* metricoptlist  */
  YYSYMBOL_metricopt = 89,                 /* metricopt  */
  YYSYMBOL_null_or_indom = 90,             /* null_or_indom  */
  YYSYMBOL_pick = 91,                      /* pick  */
  YYSYMBOL_rescaleopt = 92,                /* rescaleopt  */
  YYSYMBOL_textspec = 93,                  /* textspec  */
  YYSYMBOL_textmetricorindomspec = 94,     /* textmetricorindomspec  */
  YYSYMBOL_textmetricspec = 95,            /* textmetricspec  */
  YYSYMBOL_96_3 = 96,                      /* $@3  */
  YYSYMBOL_opttexttypes = 97,              /* opttexttypes  */
  YYSYMBOL_texttypes = 98,                 /* texttypes  */
  YYSYMBOL_texttype = 99,                  /* texttype  */
  YYSYMBOL_opttextvalue = 100,             /* opttextvalue  */
  YYSYMBOL_opttextmetricoptlist = 101,     /* opttextmetricoptlist  */
  YYSYMBOL_textmetricoptlist = 102,        /* textmetricoptlist  */
  YYSYMBOL_textmetricopt = 103,            /* textmetricopt  */
  YYSYMBOL_textstring = 104,               /* textstring  */
  YYSYMBOL_textindomspec = 105,            /* textindomspec  */
  YYSYMBOL_106_4 = 106,                    /* $@4  */
  YYSYMBOL_opttextindomoptlist = 107,      /* opttextindomoptlist  */
  YYSYMBOL_textindomoptlist = 108,         /* textindomoptlist  */
  YYSYMBOL_textindomopt = 109,             /* textindomopt  */
  YYSYMBOL_labelspec = 110,                /* labelspec  */
  YYSYMBOL_labelcontextormetricorindomspec = 111, /* labelcontextormetricorindomspec  */
  YYSYMBOL_labelcontextspec = 112,         /* labelcontextspec  */
  YYSYMBOL_113_5 = 113,                    /* $@5  */
  YYSYMBOL_optlabeldetails = 114,          /* optlabeldetails  */
  YYSYMBOL_jsonname = 115,                 /* jsonname  */
  YYSYMBOL_optlabelvalue = 116,            /* optlabelvalue  */
  YYSYMBOL_jsonvalue = 117,                /* jsonvalue  */
  YYSYMBOL_jsonnumber = 118,               /* jsonnumber  */
  YYSYMBOL_optlabelcontextoptlist = 119,   /* optlabelcontextoptlist  */
  YYSYMBOL_labelcontextoptlist = 120,      /* labelcontextoptlist  */
  YYSYMBOL_labelcontextopt = 121,          /* labelcontextopt  */
  YYSYMBOL_newlabelspec = 122,             /* newlabelspec  */
  YYSYMBOL_labeldomainspec = 123,          /* labeldomainspec  */
  YYSYMBOL_124_6 = 124,                    /* $@6  */
  YYSYMBOL_pmid_domain = 125,              /* pmid_domain  */
  YYSYMBOL_optlabeldomainoptlist = 126,    /* optlabeldomainoptlist  */
  YYSYMBOL_labeldomainoptlist = 127,       /* labeldomainoptlist  */
  YYSYMBOL_labeldomainopt = 128,           /* labeldomainopt  */
  YYSYMBOL_labelclusterspec = 129,         /* labelclusterspec  */
  YYSYMBOL_130_7 = 130,                    /* $@7  */
  YYSYMBOL_pmid_cluster = 131,             /* pmid_cluster  */
  YYSYMBOL_optlabelclusteroptlist = 132,   /* optlabelclusteroptlist  */
  YYSYMBOL_labelclusteroptlist = 133,      /* labelclusteroptlist  */
  YYSYMBOL_labelclusteropt = 134,          /* labelclusteropt  */
  YYSYMBOL_labelitemspec = 135,            /* labelitemspec  */
  YYSYMBOL_136_8 = 136,                    /* $@8  */
  YYSYMBOL_optlabelitemoptlist = 137,      /* optlabelitemoptlist  */
  YYSYMBOL_labelitemoptlist = 138,         /* labelitemoptlist  */
  YYSYMBOL_labelitemopt = 139,             /* labelitemopt  */
  YYSYMBOL_labelindomspec = 140,           /* labelindomspec  */
  YYSYMBOL_141_9 = 141,                    /* $@9  */
  YYSYMBOL_optlabelindomoptlist = 142,     /* optlabelindomoptlist  */
  YYSYMBOL_labelindomoptlist = 143,        /* labelindomoptlist  */
  YYSYMBOL_labelindomopt = 144,            /* labelindomopt  */
  YYSYMBOL_labelinstancesspec = 145,       /* labelinstancesspec  */
  YYSYMBOL_146_10 = 146,                   /* $@10  */
  YYSYMBOL_optinstancelabeldetails = 147,  /* optinstancelabeldetails  */
  YYSYMBOL_optlabelinstancesoptlist = 148, /* optlabelinstancesoptlist  */
  YYSYMBOL_labelinstancesoptlist = 149,    /* labelinstancesoptlist  */
  YYSYMBOL_labelinstancesopt = 150         /* labelinstancesopt  */
};
typedef enum yysymbol_kind_t yysymbol_kind_t;




#ifdef short
# undef short
#endif

/* On compilers that do not define __PTRDIFF_MAX__ etc., make sure
   <limits.h> and (if available) <stdint.h> are included
   so that the code can choose integer types of a good width.  */

#ifndef __PTRDIFF_MAX__
# include <limits.h> /* INFRINGES ON USER NAME SPACE */
# if defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stdint.h> /* INFRINGES ON USER NAME SPACE */
#  define YY_STDINT_H
# endif
#endif

/* Narrow types that promote to a signed type and that can represent a
   signed or unsigned integer of at least N bits.  In tables they can
   save space and decrease cache pressure.  Promoting to a signed type
   helps avoid bugs in integer arithmetic.  */

#ifdef __INT_LEAST8_MAX__
typedef __INT_LEAST8_TYPE__ yytype_int8;
#elif defined YY_STDINT_H
typedef int_least8_t yytype_int8;
#else
typedef signed char yytype_int8;
#endif

#ifdef __INT_LEAST16_MAX__
typedef __INT_LEAST16_TYPE__ yytype_int16;
#elif defined YY_STDINT_H
typedef int_least16_t yytype_int16;
#else
typedef short yytype_int16;
#endif

#if defined __UINT_LEAST8_MAX__ && __UINT_LEAST8_MAX__ <= __INT_MAX__
typedef __UINT_LEAST8_TYPE__ yytype_uint8;
#elif (!defined __UINT_LEAST8_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST8_MAX <= INT_MAX)
typedef uint_least8_t yytype_uint8;
#elif !defined __UINT_LEAST8_MAX__ && UCHAR_MAX <= INT_MAX
typedef unsigned char yytype_uint8;
#else
typedef short yytype_uint8;
#endif

#if defined __UINT_LEAST16_MAX__ && __UINT_LEAST16_MAX__ <= __INT_MAX__
typedef __UINT_LEAST16_TYPE__ yytype_uint16;
#elif (!defined __UINT_LEAST16_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST16_MAX <= INT_MAX)
typedef uint_least16_t yytype_uint16;
#elif !defined __UINT_LEAST16_MAX__ && USHRT_MAX <= INT_MAX
typedef unsigned short yytype_uint16;
#else
typedef int yytype_uint16;
#endif

#ifndef YYPTRDIFF_T
# if defined __PTRDIFF_TYPE__ && defined __PTRDIFF_MAX__
#  define YYPTRDIFF_T __PTRDIFF_TYPE__
#  define YYPTRDIFF_MAXIMUM __PTRDIFF_MAX__
# elif defined PTRDIFF_MAX
#  ifndef ptrdiff_t
#   include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  endif
#  define YYPTRDIFF_T ptrdiff_t
#  define YYPTRDIFF_MAXIMUM PTRDIFF_MAX
# else
#  define YYPTRDIFF_T long
#  define YYPTRDIFF_MAXIMUM LONG_MAX
# endif
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned
# endif
#endif

#define YYSIZE_MAXIMUM                                  \
  YY_CAST (YYPTRDIFF_T,                                 \
           (YYPTRDIFF_MAXIMUM < YY_CAST (YYSIZE_T, -1)  \
            ? YYPTRDIFF_MAXIMUM                         \
            : YY_CAST (YYSIZE_T, -1)))

#define YYSIZEOF(X) YY_CAST (YYPTRDIFF_T, sizeof (X))


/* Stored state numbers (used for stacks). */
typedef yytype_int16 yy_state_t;

/* State numbers in computations.  */
typedef int yy_state_fast_t;

#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(Msgid) Msgid
# endif
#endif


#ifndef YY_ATTRIBUTE_PURE
# if defined __GNUC__ && 2 < __GNUC__ + (96 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_PURE __attribute__ ((__pure__))
# else
#  define YY_ATTRIBUTE_PURE
# endif
#endif

#ifndef YY_ATTRIBUTE_UNUSED
# if defined __GNUC__ && 2 < __GNUC__ + (7 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_UNUSED __attribute__ ((__unused__))
# else
#  define YY_ATTRIBUTE_UNUSED
# endif
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YYUSE(E) ((void) (E))
#else
# define YYUSE(E) /* empty */
#endif

#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
/* Suppress an incorrect diagnostic about yylval being uninitialized.  */
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                            \
    _Pragma ("GCC diagnostic push")                                     \
    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
    _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
# define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
    _Pragma ("GCC diagnostic pop")
#else
# define YY_INITIAL_VALUE(Value) Value
#endif
#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_END
#endif
#ifndef YY_INITIAL_VALUE
# define YY_INITIAL_VALUE(Value) /* Nothing. */
#endif

#if defined __cplusplus && defined __GNUC__ && ! defined __ICC && 6 <= __GNUC__
# define YY_IGNORE_USELESS_CAST_BEGIN                          \
    _Pragma ("GCC diagnostic push")                            \
    _Pragma ("GCC diagnostic ignored \"-Wuseless-cast\"")
# define YY_IGNORE_USELESS_CAST_END            \
    _Pragma ("GCC diagnostic pop")
#endif
#ifndef YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_END
#endif


#define YY_ASSERT(E) ((void) (0 && (E)))

#if !defined yyoverflow

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
      /* Use EXIT_SUCCESS as a witness for stdlib.h.  */
#     ifndef EXIT_SUCCESS
#      define EXIT_SUCCESS 0
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's 'empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
       && ! ((defined YYMALLOC || defined malloc) \
             && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef EXIT_SUCCESS
#    define EXIT_SUCCESS 0
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined EXIT_SUCCESS
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined EXIT_SUCCESS
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* !defined yyoverflow */

#if (! defined yyoverflow \
     && (! defined __cplusplus \
         || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yy_state_t yyss_alloc;
  YYSTYPE yyvs_alloc;
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (YYSIZEOF (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (YYSIZEOF (yy_state_t) + YYSIZEOF (YYSTYPE)) \
      + YYSTACK_GAP_MAXIMUM)

# define YYCOPY_NEEDED 1

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack_alloc, Stack)                           \
    do                                                                  \
      {                                                                 \
        YYPTRDIFF_T yynewbytes;                                         \
        YYCOPY (&yyptr->Stack_alloc, Stack, yysize);                    \
        Stack = &yyptr->Stack_alloc;                                    \
        yynewbytes = yystacksize * YYSIZEOF (*Stack) + YYSTACK_GAP_MAXIMUM; \
        yyptr += yynewbytes / YYSIZEOF (*yyptr);                        \
      }                                                                 \
    while (0)

#endif

#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
/* Copy COUNT objects from SRC to DST.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(Dst, Src, Count) \
      __builtin_memcpy (Dst, Src, YY_CAST (YYSIZE_T, (Count)) * sizeof (*(Src)))
#  else
#   define YYCOPY(Dst, Src, Count)              \
      do                                        \
        {                                       \
          YYPTRDIFF_T yyi;                      \
          for (yyi = 0; yyi < (Count); yyi++)   \
            (Dst)[yyi] = (Src)[yyi];            \
        }                                       \
      while (0)
#  endif
# endif
#endif /* !YYCOPY_NEEDED */

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  41
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   314

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  63
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  88
/* YYNRULES -- Number of rules.  */
#define YYNRULES  233
/* YYNSTATES -- Number of states.  */
#define YYNSTATES  346

/* YYMAXUTOK -- Last valid token kind.  */
#define YYMAXUTOK   317


/* YYTRANSLATE(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex, with out-of-bounds checking.  */
#define YYTRANSLATE(YYX)                                \
  (0 <= (YYX) && (YYX) <= YYMAXUTOK                     \
   ? YY_CAST (yysymbol_kind_t, yytranslate[YYX])        \
   : YYSYMBOL_YYUNDEF)

/* YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex.  */
static const yytype_int8 yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
       5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
      15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
      25,    26,    27,    28,    29,    30,    31,    32,    33,    34,
      35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
      45,    46,    47,    48,    49,    50,    51,    52,    53,    54,
      55,    56,    57,    58,    59,    60,    61,    62
};

#if YYDEBUG
  /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
static const yytype_int16 yyrline[] =
{
       0,   891,   891,   894,   895,   898,   899,   900,   901,   902,
     905,   906,   909,   910,   913,   932,   951,   967,   972,   977,
     982,   987,   992,  1005,  1006,  1007,  1008,  1011,  1016,  1021,
    1028,  1035,  1042,  1043,  1044,  1047,  1060,  1072,  1085,  1097,
    1106,  1118,  1117,  1150,  1157,  1179,  1193,  1194,  1197,  1198,
    1201,  1226,  1236,  1245,  1253,  1261,  1266,  1271,  1276,  1281,
    1286,  1291,  1296,  1303,  1304,  1308,  1307,  1350,  1357,  1358,
    1376,  1404,  1434,  1435,  1438,  1439,  1442,  1466,  1490,  1518,
    1549,  1623,  1640,  1686,  1693,  1698,  1703,  1708,  1713,  1718,
    1723,  1728,  1733,  1738,  1743,  1750,  1751,  1757,  1763,  1769,
    1773,  1778,  1781,  1783,  1786,  1789,  1790,  1794,  1793,  1893,
    1896,  1899,  1901,  1905,  1907,  1909,  1913,  1916,  1919,  1921,
    1929,  1930,  1933,  1940,  1969,  2018,  2020,  2025,  2024,  2117,
    2119,  2127,  2128,  2131,  2138,  2167,  2195,  2198,  2199,  2200,
    2201,  2202,  2203,  2207,  2206,  2261,  2263,  2266,  2272,  2274,
    2281,  2283,  2286,  2289,  2291,  2293,  2295,  2297,  2301,  2306,
    2308,  2313,  2317,  2319,  2324,  2325,  2328,  2335,  2359,  2383,
    2388,  2404,  2403,  2463,  2483,  2485,  2490,  2491,  2494,  2501,
    2527,  2551,  2575,  2582,  2581,  2660,  2683,  2703,  2705,  2710,
    2711,  2714,  2721,  2747,  2771,  2795,  2802,  2801,  2891,  2893,
    2898,  2899,  2902,  2909,  2937,  2961,  2985,  2992,  2991,  3074,
    3076,  3081,  3082,  3085,  3092,  3118,  3142,  3166,  3173,  3172,
    3257,  3262,  3268,  3274,  3282,  3284,  3289,  3290,  3293,  3300,
    3326,  3351,  3375,  3399
};
#endif

/** Accessing symbol of state STATE.  */
#define YY_ACCESSING_SYMBOL(State) YY_CAST (yysymbol_kind_t, yystos[State])

#if YYDEBUG || 0
/* The user-facing name of the symbol whose (internal) number is
   YYSYMBOL.  No bounds checking.  */
static const char *yysymbol_name (yysymbol_kind_t yysymbol) YY_ATTRIBUTE_UNUSED;

/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "\"end of file\"", "error", "\"invalid token\"", "TOK_LBRACE",
  "TOK_RBRACE", "TOK_PLUS", "TOK_MINUS", "TOK_COLON", "TOK_COMMA",
  "TOK_ASSIGN", "TOK_GLOBAL", "TOK_INDOM", "TOK_DUPLICATE", "TOK_METRIC",
  "TOK_HOSTNAME", "TOK_TZ", "TOK_TIME", "TOK_NAME", "TOK_INST",
  "TOK_INAME", "TOK_DELETE", "TOK_PMID", "TOK_NULL_INT", "TOK_TYPE",
  "TOK_IF", "TOK_SEM", "TOK_UNITS", "TOK_OUTPUT", "TOK_RESCALE",
  "TOK_TEXT", "TOK_ONELINE", "TOK_HELP", "TOK_TEXT_STAR", "TOK_LABEL",
  "TOK_CONTEXT", "TOK_LABEL_STAR", "TOK_DOMAIN", "TOK_CLUSTER", "TOK_ITEM",
  "TOK_INSTANCES", "TOK_INSTANCE", "TOK_NEW", "TOK_VALUE", "TOK_GNAME",
  "TOK_NUMBER", "TOK_STRING", "TOK_TEXT_STRING", "TOK_HNAME", "TOK_FLOAT",
  "TOK_JSON_STRING", "TOK_JSON_NUMBER", "TOK_JSON_TRUE", "TOK_JSON_FALSE",
  "TOK_JSON_NULL", "TOK_INDOM_STAR", "TOK_PMID_INT", "TOK_PMID_STAR",
  "TOK_TYPE_NAME", "TOK_SEM_NAME", "TOK_SPACE_NAME", "TOK_TIME_NAME",
  "TOK_COUNT_NAME", "TOK_OUTPUT_TYPE", "$accept", "config", "speclist",
  "spec", "globalspec", "globaloptlist", "globalopt", "hname",
  "signnumber", "number", "float", "signtime", "time", "indomspec", "$@1",
  "indom_int", "optindomopt", "indomoptlist", "indomopt", "duplicateopt",
  "metricspec", "$@2", "pmid_or_name", "pmid_int", "optmetricoptlist",
  "metricoptlist", "metricopt", "null_or_indom", "pick", "rescaleopt",
  "textspec", "textmetricorindomspec", "textmetricspec", "$@3",
  "opttexttypes", "texttypes", "texttype", "opttextvalue",
  "opttextmetricoptlist", "textmetricoptlist", "textmetricopt",
  "textstring", "textindomspec", "$@4", "opttextindomoptlist",
  "textindomoptlist", "textindomopt", "labelspec",
  "labelcontextormetricorindomspec", "labelcontextspec", "$@5",
  "optlabeldetails", "jsonname", "optlabelvalue", "jsonvalue",
  "jsonnumber", "optlabelcontextoptlist", "labelcontextoptlist",
  "labelcontextopt", "newlabelspec", "labeldomainspec", "$@6",
  "pmid_domain", "optlabeldomainoptlist", "labeldomainoptlist",
  "labeldomainopt", "labelclusterspec", "$@7", "pmid_cluster",
  "optlabelclusteroptlist", "labelclusteroptlist", "labelclusteropt",
  "labelitemspec", "$@8", "optlabelitemoptlist", "labelitemoptlist",
  "labelitemopt", "labelindomspec", "$@9", "optlabelindomoptlist",
  "labelindomoptlist", "labelindomopt", "labelinstancesspec", "$@10",
  "optinstancelabeldetails", "optlabelinstancesoptlist",
  "labelinstancesoptlist", "labelinstancesopt", YY_NULLPTR
};

static const char *
yysymbol_name (yysymbol_kind_t yysymbol)
{
  return yytname[yysymbol];
}
#endif

#ifdef YYPRINT
/* YYTOKNUM[NUM] -- (External) token number corresponding to the
   (internal) symbol number NUM (which must be that of a token).  */
static const yytype_int16 yytoknum[] =
{
       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
     275,   276,   277,   278,   279,   280,   281,   282,   283,   284,
     285,   286,   287,   288,   289,   290,   291,   292,   293,   294,
     295,   296,   297,   298,   299,   300,   301,   302,   303,   304,
     305,   306,   307,   308,   309,   310,   311,   312,   313,   314,
     315,   316,   317
};
#endif

#define YYPACT_NINF (-255)

#define yypact_value_is_default(Yyn) \
  ((Yyn) == YYPACT_NINF)

#define YYTABLE_NINF (-65)

#define yytable_value_is_error(Yyn) \
  0

  /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
     STATE-NUM.  */
static const yytype_int16 yypact[] =
{
     110,    74,   -25,    31,   116,    80,    22,  -255,   110,  -255,
    -255,  -255,  -255,  -255,   181,  -255,  -255,  -255,  -255,  -255,
    -255,  -255,  -255,   -25,    31,  -255,  -255,  -255,   -25,    47,
       0,    -9,    31,   -25,  -255,  -255,  -255,  -255,  -255,  -255,
    -255,  -255,  -255,  -255,    85,    88,   101,    98,   190,   112,
     133,   177,   177,    47,     5,  -255,  -255,  -255,     5,  -255,
      47,  -255,  -255,    47,    47,   124,   155,    96,    36,  -255,
    -255,    15,   167,  -255,  -255,  -255,   118,  -255,   177,   118,
    -255,    17,  -255,  -255,  -255,  -255,  -255,  -255,  -255,  -255,
    -255,  -255,   147,  -255,  -255,  -255,  -255,    47,    47,    47,
    -255,  -255,  -255,  -255,  -255,  -255,  -255,    82,    82,  -255,
    -255,   146,  -255,  -255,  -255,   152,   135,   141,   151,  -255,
      15,   182,   185,  -255,   191,    19,   192,   203,   209,  -255,
     167,  -255,  -255,  -255,  -255,  -255,  -255,   211,  -255,  -255,
     132,   212,   213,   214,  -255,  -255,  -255,   215,  -255,  -255,
      82,    25,   210,   216,  -255,  -255,    41,   178,   127,   163,
     165,   166,     9,  -255,  -255,   220,   223,    70,  -255,   218,
      83,   219,   225,  -255,   132,  -255,   115,   129,   134,   105,
     224,  -255,  -255,   -25,     4,     1,  -255,  -255,   205,  -255,
    -255,  -255,   221,  -255,   189,   193,  -255,   226,    93,   111,
     227,  -255,   229,   230,  -255,   231,  -255,    70,    83,    56,
      56,  -255,  -255,  -255,   232,   233,   235,  -255,   236,  -255,
     115,  -255,   237,   239,   240,  -255,   241,  -255,   129,  -255,
     242,   243,   244,  -255,   250,  -255,   134,  -255,   247,   248,
     249,   252,  -255,   255,  -255,   105,    82,  -255,  -255,  -255,
    -255,  -255,   -10,  -255,   206,  -255,  -255,     9,   253,  -255,
     257,   260,  -255,    93,   258,  -255,   259,   265,  -255,   111,
     -25,    83,    56,  -255,  -255,  -255,  -255,  -255,    83,   228,
      56,  -255,  -255,    83,    -9,    56,  -255,  -255,    83,   127,
      56,  -255,  -255,    83,   -25,   234,    56,  -255,  -255,  -255,
    -255,   135,   245,  -255,  -255,   262,   -25,   118,  -255,  -255,
     127,   118,  -255,  -255,  -255,  -255,  -255,  -255,  -255,  -255,
    -255,  -255,  -255,  -255,  -255,  -255,  -255,  -255,  -255,  -255,
    -255,  -255,     9,  -255,  -255,  -255,  -255,   266,   217,   267,
     222,   269,   238,   251,  -255,  -255
};

  /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
     Performed when YYTABLE does not specify something else to do.  Zero
     means the default is an error.  */
static const yytype_uint8 yydefact[] =
{
       0,     0,    43,    67,     0,     0,     0,     2,     3,     5,
       6,     7,     8,     9,     0,    44,    45,    41,    69,    70,
      71,    65,    68,     0,     0,   104,   105,   106,     0,   147,
       0,     0,     0,     0,   136,   137,   138,   139,   140,   141,
     142,     1,     4,    11,    18,    20,    22,     0,    12,     0,
       0,   110,   110,   147,   152,   149,   148,   143,   152,   173,
     147,   185,   186,   147,   147,   223,    17,    19,    21,    10,
      13,    47,    73,   115,   114,   113,   117,   109,   111,   117,
     207,     0,   151,   159,   161,   155,   156,   157,   153,   146,
     150,   154,     0,   145,   171,   183,   196,   147,   147,   147,
     218,    24,    25,    23,    26,    14,    15,     0,     0,    30,
      31,    40,    39,    16,    34,    56,    62,    59,     0,    46,
      48,     0,     0,    83,     0,     0,     0,     0,     0,    72,
      74,   125,   126,   127,   116,   112,   107,     0,   158,   160,
     163,     0,     0,     0,   222,   221,   220,     0,    32,    33,
       0,    55,    61,    58,    42,    49,    89,    85,    84,    86,
      87,    90,    91,    66,    75,     0,     0,   210,   166,     0,
       0,     0,     0,   162,   164,   169,   175,   188,   199,   225,
      38,    37,    63,     0,    60,    57,    96,    95,   101,    77,
      76,    78,     0,    81,     0,     0,    29,     0,   130,   119,
       0,   213,     0,     0,   217,     0,   209,   211,     0,     0,
       0,   144,   165,   178,     0,     0,     0,   182,     0,   174,
     176,   191,     0,     0,     0,   195,     0,   187,   189,   202,
       0,     0,     0,   206,     0,   198,   200,   228,     0,     0,
       0,     0,   233,     0,   224,   226,     0,    50,    54,    53,
      52,    51,   100,    80,    88,    27,    28,     0,     0,   133,
       0,     0,   129,   131,     0,   122,     0,     0,   118,   120,
       0,     0,     0,   208,   212,   167,   170,   168,     0,     0,
       0,   172,   177,     0,     0,     0,   184,   190,     0,     0,
       0,   197,   201,     0,     0,     0,     0,   219,   227,    36,
      35,     0,     0,    99,    79,     0,     0,     0,   128,   132,
       0,     0,   108,   121,   214,   215,   216,   180,   179,   181,
     193,   192,   194,   204,   203,   205,   231,   229,   230,   232,
      97,    98,     0,   135,   134,   124,   123,     0,    92,     0,
      93,     0,    94,   103,   102,    82
};

  /* YYPGOTO[NTERM-NUM].  */
static const yytype_int16 yypgoto[] =
{
    -255,  -255,   272,  -255,  -255,   246,  -255,  -255,  -254,  -112,
    -148,  -255,   103,  -255,  -255,   -23,  -255,   153,  -255,  -255,
    -255,  -255,    27,  -152,  -255,   154,  -255,  -255,  -255,  -255,
    -255,  -255,  -255,  -255,   254,   207,  -255,   202,  -255,    18,
    -255,  -130,  -255,  -255,  -255,    23,  -255,  -255,  -255,  -255,
    -255,   -33,   -28,   256,  -197,  -255,  -255,   114,  -255,  -160,
    -255,  -255,  -255,  -255,    69,  -255,  -255,  -255,     7,  -255,
      64,  -255,  -255,  -255,  -255,    57,  -255,  -255,  -255,  -255,
      89,  -255,  -255,  -255,  -255,  -255,    50,  -255
};

  /* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int16 yydefgoto[] =
{
      -1,     6,     7,     8,     9,    47,    48,   105,   197,   111,
     112,   113,   114,    10,    49,    17,   118,   119,   120,   183,
      11,    50,    21,    22,   128,   129,   130,   188,   253,   345,
      12,    25,    26,   166,    76,    77,    78,   133,   267,   268,
     269,   134,    27,   165,   261,   262,   263,    13,    34,    35,
      92,    57,    88,    89,    90,    91,   172,   173,   174,   175,
      36,   141,    60,   218,   219,   220,    37,   142,    63,   226,
     227,   228,    38,   143,   234,   235,   236,    39,   137,   205,
     206,   207,    40,   147,   100,   243,   244,   245
};

  /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
     positive, shift that token.  If negative, reduce the rule whose
     number is the opposite.  If YYTABLE_NINF, syntax error.  */
static const yytype_int16 yytable[] =
{
      51,    58,   181,   305,   152,    53,   190,   204,   301,   302,
      65,    81,   276,   277,   194,   195,   217,   225,   233,   242,
      80,   250,    41,    15,   248,    58,   115,    94,   159,    16,
      95,    96,    58,   116,   117,    58,    58,   182,   180,    61,
      82,   107,   108,   160,    59,    62,   251,   204,   109,    83,
      55,    52,   303,   196,    56,    84,    85,    86,    87,    64,
     217,   138,    81,   186,   144,   145,   146,   139,   225,    58,
      58,    58,   249,   -64,    18,   316,   233,    14,   337,   -64,
     109,   200,    54,   319,   110,   242,    19,    20,   322,    15,
     201,    28,    55,   325,    66,    16,    56,    67,   300,   329,
      83,    55,    69,   202,   258,    56,    84,    85,    86,    87,
      68,   170,   203,   259,    29,    71,    30,    31,    32,    33,
       1,     2,   260,     3,   264,   237,   109,    23,    55,    24,
     110,   265,    56,   187,   299,   213,    72,   324,   238,     4,
     266,   106,   209,     5,   239,   240,   170,   241,   214,   221,
     140,   215,   168,   150,   229,   154,   170,   216,   335,    97,
     247,   151,   222,   131,   132,   169,   223,   230,    98,    99,
     170,   224,   231,   170,   171,   170,   232,   334,   121,   109,
     275,   336,    19,    20,   122,    43,   153,   123,   124,   330,
     125,   156,   126,   127,   157,    44,    45,    46,   101,   102,
     158,   161,   103,   104,    44,    45,    46,    73,    74,    75,
     148,   149,   162,   163,   167,   176,   177,   178,   179,   184,
     191,   189,   192,   198,   193,   185,   199,   208,   210,   211,
     254,   246,   252,   255,   257,   273,   270,   256,   271,   272,
     281,   278,   279,   315,   280,   286,   283,   314,   284,   285,
     317,   288,   289,   290,   291,   320,   293,   294,   295,   297,
     323,   296,   306,   304,   308,   326,   307,   310,   311,   312,
     332,   327,   318,   155,   338,   340,   339,   342,   328,   344,
      42,   136,   341,   333,   164,   135,   309,   313,   212,   282,
     331,   321,   287,   292,    70,   298,   274,     0,     0,   343,
       0,     0,     0,     0,     0,     0,    79,     0,     0,     0,
       0,     0,     0,     0,    93
};

static const yytype_int16 yycheck[] =
{
      23,    29,   150,   257,   116,    28,   158,   167,    18,    19,
      33,     6,   209,   210,     5,     6,   176,   177,   178,   179,
      53,    20,     0,    48,    20,    53,    11,    60,     9,    54,
      63,    64,    60,    18,    19,    63,    64,    12,   150,    48,
      35,     5,     6,    24,    44,    54,    45,   207,    44,    44,
      45,    24,    62,    44,    49,    50,    51,    52,    53,    32,
     220,    44,     6,    22,    97,    98,    99,    50,   228,    97,
      98,    99,   184,    48,    43,   272,   236,     3,   332,    54,
      44,    11,    35,   280,    48,   245,    55,    56,   285,    48,
      20,    11,    45,   290,     9,    54,    49,     9,   246,   296,
      44,    45,     4,    33,    11,    49,    50,    51,    52,    53,
       9,    41,    42,    20,    34,     3,    36,    37,    38,    39,
      10,    11,    29,    13,    13,    20,    44,    11,    45,    13,
      48,    20,    49,   156,   246,    20,     3,   289,    33,    29,
      29,    45,   170,    33,    39,    40,    41,    42,    33,    20,
       3,    36,    20,     7,    20,     4,    41,    42,   310,    35,
     183,     9,    33,    45,    46,    33,    37,    33,    44,    45,
      41,    42,    38,    41,    42,    41,    42,   307,    11,    44,
     208,   311,    55,    56,    17,     4,    45,    20,    21,   301,
      23,     9,    25,    26,     9,    14,    15,    16,    43,    44,
       9,     9,    47,    48,    14,    15,    16,    30,    31,    32,
     107,   108,     9,     4,     3,     3,     3,     3,     3,     9,
      57,    43,    57,     3,    58,     9,     3,     9,     9,     4,
       9,     7,    27,    44,     8,     4,     9,    44,     9,     9,
       4,     9,     9,   271,     9,     4,     9,   270,     9,     9,
     278,     9,     9,     9,     4,   283,     9,     9,     9,     4,
     288,     9,     9,    57,     4,   293,     9,     9,     9,     4,
       8,   294,    44,   120,     8,     8,    59,     8,    44,    28,
       8,    79,    60,   306,   130,    78,   263,   269,   174,   220,
      45,   284,   228,   236,    48,   245,   207,    -1,    -1,    61,
      -1,    -1,    -1,    -1,    -1,    -1,    52,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    58
};

  /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
     symbol of state STATE-NUM.  */
static const yytype_uint8 yystos[] =
{
       0,    10,    11,    13,    29,    33,    64,    65,    66,    67,
      76,    83,    93,   110,     3,    48,    54,    78,    43,    55,
      56,    85,    86,    11,    13,    94,    95,   105,    11,    34,
      36,    37,    38,    39,   111,   112,   123,   129,   135,   140,
     145,     0,    65,     4,    14,    15,    16,    68,    69,    77,
      84,    78,    85,    78,    35,    45,    49,   114,   115,    44,
     125,    48,    54,   131,    85,    78,     9,     9,     9,     4,
      68,     3,     3,    30,    31,    32,    97,    98,    99,    97,
     114,     6,    35,    44,    50,    51,    52,    53,   115,   116,
     117,   118,   113,   116,   114,   114,   114,    35,    44,    45,
     147,    43,    44,    47,    48,    70,    45,     5,     6,    44,
      48,    72,    73,    74,    75,    11,    18,    19,    79,    80,
      81,    11,    17,    20,    21,    23,    25,    26,    87,    88,
      89,    45,    46,   100,   104,    98,   100,   141,    44,    50,
       3,   124,   130,   136,   114,   114,   114,   146,    75,    75,
       7,     9,    72,    45,     4,    80,     9,     9,     9,     9,
      24,     9,     9,     4,    88,   106,    96,     3,    20,    33,
      41,    42,   119,   120,   121,   122,     3,     3,     3,     3,
      72,    73,    12,    82,     9,     9,    22,    78,    90,    43,
      86,    57,    57,    58,     5,     6,    44,    71,     3,     3,
      11,    20,    33,    42,   122,   142,   143,   144,     9,   115,
       9,     4,   120,    20,    33,    36,    42,   122,   126,   127,
     128,    20,    33,    37,    42,   122,   132,   133,   134,    20,
      33,    38,    42,   122,   137,   138,   139,    20,    33,    39,
      40,    42,   122,   148,   149,   150,     7,    78,    20,    72,
      20,    45,    27,    91,     9,    44,    44,     8,    11,    20,
      29,   107,   108,   109,    13,    20,    29,   101,   102,   103,
       9,     9,     9,     4,   143,   115,   117,   117,     9,     9,
       9,     4,   127,     9,     9,     9,     4,   133,     9,     9,
       9,     4,   138,     9,     9,     9,     9,     4,   149,    72,
      73,    18,    19,    62,    57,    71,     9,     9,     4,   108,
       9,     9,     4,   102,    78,   115,   117,   115,    44,   117,
     115,   131,   117,   115,    86,   117,   115,    78,    44,   117,
      72,    45,     8,    78,   104,    86,   104,    71,     8,    59,
       8,    60,     8,    61,    28,    92
};

  /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
static const yytype_uint8 yyr1[] =
{
       0,    63,    64,    65,    65,    66,    66,    66,    66,    66,
      67,    67,    68,    68,    69,    69,    69,    69,    69,    69,
      69,    69,    69,    70,    70,    70,    70,    71,    71,    71,
      72,    73,    74,    74,    74,    75,    75,    75,    75,    75,
      75,    77,    76,    76,    78,    78,    79,    79,    80,    80,
      81,    81,    81,    81,    81,    81,    81,    81,    81,    81,
      81,    81,    81,    82,    82,    84,    83,    83,    85,    85,
      86,    86,    87,    87,    88,    88,    89,    89,    89,    89,
      89,    89,    89,    89,    89,    89,    89,    89,    89,    89,
      89,    89,    89,    89,    89,    90,    90,    91,    91,    91,
      91,    91,    92,    92,    93,    94,    94,    96,    95,    97,
      97,    98,    98,    99,    99,    99,   100,   100,   101,   101,
     102,   102,   103,   103,   103,   104,   104,   106,   105,   107,
     107,   108,   108,   109,   109,   109,   110,   111,   111,   111,
     111,   111,   111,   113,   112,   114,   114,   114,   115,   115,
     116,   116,   116,   117,   117,   117,   117,   117,   118,   118,
     118,   118,   119,   119,   120,   120,   121,   121,   121,   121,
     122,   124,   123,   125,   126,   126,   127,   127,   128,   128,
     128,   128,   128,   130,   129,   131,   131,   132,   132,   133,
     133,   134,   134,   134,   134,   134,   136,   135,   137,   137,
     138,   138,   139,   139,   139,   139,   139,   141,   140,   142,
     142,   143,   143,   144,   144,   144,   144,   144,   146,   145,
     147,   147,   147,   147,   148,   148,   149,   149,   150,   150,
     150,   150,   150,   150
};

  /* YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.  */
static const yytype_int8 yyr2[] =
{
       0,     2,     1,     1,     2,     1,     1,     1,     1,     1,
       4,     3,     1,     2,     3,     3,     3,     2,     1,     2,
       1,     2,     1,     1,     1,     1,     1,     2,     2,     1,
       1,     1,     2,     2,     1,     5,     5,     3,     3,     1,
       1,     0,     6,     1,     1,     1,     1,     0,     1,     2,
       4,     4,     4,     4,     4,     2,     1,     3,     2,     1,
       3,     2,     1,     1,     0,     0,     6,     1,     1,     1,
       1,     1,     1,     0,     1,     2,     3,     3,     3,     5,
       4,     3,    14,     1,     2,     2,     2,     2,     4,     2,
       2,     2,     8,    10,    12,     1,     1,     3,     3,     2,
       1,     0,     1,     0,     2,     1,     1,     0,     8,     1,
       0,     1,     2,     1,     1,     1,     1,     0,     1,     0,
       1,     2,     1,     3,     3,     1,     1,     0,     8,     1,
       0,     1,     2,     1,     3,     3,     2,     1,     1,     1,
       1,     1,     1,     0,     6,     2,     2,     0,     1,     1,
       1,     1,     0,     1,     1,     1,     1,     1,     2,     1,
       2,     1,     1,     0,     1,     2,     1,     3,     3,     1,
       3,     0,     7,     1,     1,     0,     1,     2,     1,     3,
       3,     3,     1,     0,     7,     1,     1,     1,     0,     1,
       2,     1,     3,     3,     3,     1,     0,     7,     1,     0,
       1,     2,     1,     3,     3,     3,     1,     0,     7,     1,
       0,     1,     2,     1,     3,     3,     3,     1,     0,     7,
       2,     2,     2,     0,     1,     0,     1,     2,     1,     3,
       3,     3,     3,     1
};


enum { YYENOMEM = -2 };

#define yyerrok         (yyerrstatus = 0)
#define yyclearin       (yychar = YYEMPTY)

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab


#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)                                    \
  do                                                              \
    if (yychar == YYEMPTY)                                        \
      {                                                           \
        yychar = (Token);                                         \
        yylval = (Value);                                         \
        YYPOPSTACK (yylen);                                       \
        yystate = *yyssp;                                         \
        goto yybackup;                                            \
      }                                                           \
    else                                                          \
      {                                                           \
        yyerror (YY_("syntax error: cannot back up")); \
        YYERROR;                                                  \
      }                                                           \
  while (0)

/* Backward compatibility with an undocumented macro.
   Use YYerror or YYUNDEF. */
#define YYERRCODE YYUNDEF


/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)                        \
do {                                            \
  if (yydebug)                                  \
    YYFPRINTF Args;                             \
} while (0)

/* This macro is provided for backward compatibility. */
# ifndef YY_LOCATION_PRINT
#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
# endif


# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)                    \
do {                                                                      \
  if (yydebug)                                                            \
    {                                                                     \
      YYFPRINTF (stderr, "%s ", Title);                                   \
      yy_symbol_print (stderr,                                            \
                  Kind, Value); \
      YYFPRINTF (stderr, "\n");                                           \
    }                                                                     \
} while (0)


/*-----------------------------------.
| Print this symbol's value on YYO.  |
`-----------------------------------*/

static void
yy_symbol_value_print (FILE *yyo,
                       yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep)
{
  FILE *yyoutput = yyo;
  YYUSE (yyoutput);
  if (!yyvaluep)
    return;
# ifdef YYPRINT
  if (yykind < YYNTOKENS)
    YYPRINT (yyo, yytoknum[yykind], *yyvaluep);
# endif
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YYUSE (yykind);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}


/*---------------------------.
| Print this symbol on YYO.  |
`---------------------------*/

static void
yy_symbol_print (FILE *yyo,
                 yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep)
{
  YYFPRINTF (yyo, "%s %s (",
             yykind < YYNTOKENS ? "token" : "nterm", yysymbol_name (yykind));

  yy_symbol_value_print (yyo, yykind, yyvaluep);
  YYFPRINTF (yyo, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

static void
yy_stack_print (yy_state_t *yybottom, yy_state_t *yytop)
{
  YYFPRINTF (stderr, "Stack now");
  for (; yybottom <= yytop; yybottom++)
    {
      int yybot = *yybottom;
      YYFPRINTF (stderr, " %d", yybot);
    }
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)                            \
do {                                                            \
  if (yydebug)                                                  \
    yy_stack_print ((Bottom), (Top));                           \
} while (0)


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

static void
yy_reduce_print (yy_state_t *yyssp, YYSTYPE *yyvsp,
                 int yyrule)
{
  int yylno = yyrline[yyrule];
  int yynrhs = yyr2[yyrule];
  int yyi;
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %d):\n",
             yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr,
                       YY_ACCESSING_SYMBOL (+yyssp[yyi + 1 - yynrhs]),
                       &yyvsp[(yyi + 1) - (yynrhs)]);
      YYFPRINTF (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)          \
do {                                    \
  if (yydebug)                          \
    yy_reduce_print (yyssp, yyvsp, Rule); \
} while (0)

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args) ((void) 0)
# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !YYDEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif






/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

static void
yydestruct (const char *yymsg,
            yysymbol_kind_t yykind, YYSTYPE *yyvaluep)
{
  YYUSE (yyvaluep);
  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yykind, yyvaluep, yylocationp);

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YYUSE (yykind);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}


/* Lookahead token kind.  */
int yychar;

/* The semantic value of the lookahead symbol.  */
YYSTYPE yylval;
/* Number of syntax errors so far.  */
int yynerrs;




/*----------.
| yyparse.  |
`----------*/

int
yyparse (void)
{
    yy_state_fast_t yystate = 0;
    /* Number of tokens to shift before error messages enabled.  */
    int yyerrstatus = 0;

    /* Refer to the stacks through separate pointers, to allow yyoverflow
       to reallocate them elsewhere.  */

    /* Their size.  */
    YYPTRDIFF_T yystacksize = YYINITDEPTH;

    /* The state stack: array, bottom, top.  */
    yy_state_t yyssa[YYINITDEPTH];
    yy_state_t *yyss = yyssa;
    yy_state_t *yyssp = yyss;

    /* The semantic value stack: array, bottom, top.  */
    YYSTYPE yyvsa[YYINITDEPTH];
    YYSTYPE *yyvs = yyvsa;
    YYSTYPE *yyvsp = yyvs;

  int yyn;
  /* The return value of yyparse.  */
  int yyresult;
  /* Lookahead symbol kind.  */
  yysymbol_kind_t yytoken = YYSYMBOL_YYEMPTY;
  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;



#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))

  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yychar = YYEMPTY; /* Cause a token to be read.  */
  goto yysetstate;


/*------------------------------------------------------------.
| yynewstate -- push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;


/*--------------------------------------------------------------------.
| yysetstate -- set current state (the top of the stack) to yystate.  |
`--------------------------------------------------------------------*/
yysetstate:
  YYDPRINTF ((stderr, "Entering state %d\n", yystate));
  YY_ASSERT (0 <= yystate && yystate < YYNSTATES);
  YY_IGNORE_USELESS_CAST_BEGIN
  *yyssp = YY_CAST (yy_state_t, yystate);
  YY_IGNORE_USELESS_CAST_END
  YY_STACK_PRINT (yyss, yyssp);

  if (yyss + yystacksize - 1 <= yyssp)
#if !defined yyoverflow && !defined YYSTACK_RELOCATE
    goto yyexhaustedlab;
#else
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYPTRDIFF_T yysize = yyssp - yyss + 1;

# if defined yyoverflow
      {
        /* Give user a chance to reallocate the stack.  Use copies of
           these so that the &'s don't force the real ones into
           memory.  */
        yy_state_t *yyss1 = yyss;
        YYSTYPE *yyvs1 = yyvs;

        /* Each stack pointer address is followed by the size of the
           data in use in that stack, in bytes.  This used to be a
           conditional around just the two extra args, but that might
           be undefined if yyoverflow is a macro.  */
        yyoverflow (YY_("memory exhausted"),
                    &yyss1, yysize * YYSIZEOF (*yyssp),
                    &yyvs1, yysize * YYSIZEOF (*yyvsp),
                    &yystacksize);
        yyss = yyss1;
        yyvs = yyvs1;
      }
# else /* defined YYSTACK_RELOCATE */
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
        goto yyexhaustedlab;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
        yystacksize = YYMAXDEPTH;

      {
        yy_state_t *yyss1 = yyss;
        union yyalloc *yyptr =
          YY_CAST (union yyalloc *,
                   YYSTACK_ALLOC (YY_CAST (YYSIZE_T, YYSTACK_BYTES (yystacksize))));
        if (! yyptr)
          goto yyexhaustedlab;
        YYSTACK_RELOCATE (yyss_alloc, yyss);
        YYSTACK_RELOCATE (yyvs_alloc, yyvs);
#  undef YYSTACK_RELOCATE
        if (yyss1 != yyssa)
          YYSTACK_FREE (yyss1);
      }
# endif

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;

      YY_IGNORE_USELESS_CAST_BEGIN
      YYDPRINTF ((stderr, "Stack size increased to %ld\n",
                  YY_CAST (long, yystacksize)));
      YY_IGNORE_USELESS_CAST_END

      if (yyss + yystacksize - 1 <= yyssp)
        YYABORT;
    }
#endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */

  if (yystate == YYFINAL)
    YYACCEPT;

  goto yybackup;


/*-----------.
| yybackup.  |
`-----------*/
yybackup:
  /* Do appropriate processing given the current state.  Read a
     lookahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to lookahead token.  */
  yyn = yypact[yystate];
  if (yypact_value_is_default (yyn))
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* YYCHAR is either empty, or end-of-input, or a valid lookahead.  */
  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token\n"));
      yychar = yylex ();
    }

  if (yychar <= YYEOF)
    {
      yychar = YYEOF;
      yytoken = YYSYMBOL_YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else if (yychar == YYerror)
    {
      /* The scanner already issued an error message, process directly
         to error recovery.  But do not keep the error token as
         lookahead, it is too special and may lead us to an endless
         loop in error recovery. */
      yychar = YYUNDEF;
      yytoken = YYSYMBOL_YYerror;
      goto yyerrlab1;
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yytable_value_is_error (yyn))
        goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the lookahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
  yystate = yyn;
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END

  /* Discard the shifted token.  */
  yychar = YYEMPTY;
  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     '$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];


  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
  case 14: /* globalopt: TOK_HOSTNAME TOK_ASSIGN hname  */
#line 914 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_HOSTNAME) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global hostname clause");
			    yyerror(mess);
			}
			if (strcmp(inarch.label.ll_hostname, (yyvsp[0].str)) == 0) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global hostname (%s): No change", inarch.label.ll_hostname);
				yywarn(mess);
			    }
			}
			else {
			    strncpy(global.hostname, (yyvsp[0].str), sizeof(global.hostname)-1);
			    global.flags |= GLOBAL_CHANGE_HOSTNAME;
			}
			free((yyvsp[0].str));
		    }
#line 2310 "gram.tab.c"
    break;

  case 15: /* globalopt: TOK_TZ TOK_ASSIGN TOK_STRING  */
#line 933 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_TZ) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global tz clause");
			    yyerror(mess);
			}
			if (strcmp(inarch.label.ll_tz, (yyvsp[0].str)) == 0) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global timezone (%s): No change", inarch.label.ll_tz);
				yywarn(mess);
			    }
			}
			else {
			    strncpy(global.tz, (yyvsp[0].str), sizeof(global.tz)-1);
			    global.flags |= GLOBAL_CHANGE_TZ;
			}
			free((yyvsp[0].str));
		    }
#line 2333 "gram.tab.c"
    break;

  case 16: /* globalopt: TOK_TIME TOK_ASSIGN signtime  */
#line 952 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_TIME) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global time clause");
			    yyerror(mess);
			}
			if (global.time.sec == 0 && global.time.nsec == 0) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global time: No change");
				yywarn(mess);
			    }
			}
			else
			    global.flags |= GLOBAL_CHANGE_TIME;
		    }
#line 2353 "gram.tab.c"
    break;

  case 17: /* globalopt: TOK_HOSTNAME TOK_ASSIGN  */
#line 968 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting hostname in hostname clause");
			yyerror(mess);
		    }
#line 2362 "gram.tab.c"
    break;

  case 18: /* globalopt: TOK_HOSTNAME  */
#line 973 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in hostname clause");
			yyerror(mess);
		    }
#line 2371 "gram.tab.c"
    break;

  case 19: /* globalopt: TOK_TZ TOK_ASSIGN  */
#line 978 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting timezone string in tz clause");
			yyerror(mess);
		    }
#line 2380 "gram.tab.c"
    break;

  case 20: /* globalopt: TOK_TZ  */
#line 983 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in tz clause");
			yyerror(mess);
		    }
#line 2389 "gram.tab.c"
    break;

  case 21: /* globalopt: TOK_TIME TOK_ASSIGN  */
#line 988 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting delta of the form [+-][HH:[MM:]]SS[.d...] in time clause");
			yyerror(mess);
		    }
#line 2398 "gram.tab.c"
    break;

  case 22: /* globalopt: TOK_TIME  */
#line 993 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in time clause");
			yyerror(mess);
		    }
#line 2407 "gram.tab.c"
    break;

  case 27: /* signnumber: TOK_PLUS TOK_NUMBER  */
#line 1012 "gram.y"
                    {
			(yyval.ival) = atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2416 "gram.tab.c"
    break;

  case 28: /* signnumber: TOK_MINUS TOK_NUMBER  */
#line 1017 "gram.y"
                    {
			(yyval.ival) = -atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2425 "gram.tab.c"
    break;

  case 29: /* signnumber: TOK_NUMBER  */
#line 1022 "gram.y"
                    {
			(yyval.ival) = atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2434 "gram.tab.c"
    break;

  case 30: /* number: TOK_NUMBER  */
#line 1029 "gram.y"
                    {
			(yyval.ival) = atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2443 "gram.tab.c"
    break;

  case 31: /* float: TOK_FLOAT  */
#line 1036 "gram.y"
                    {
			(yyval.dval) = atof((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2452 "gram.tab.c"
    break;

  case 33: /* signtime: TOK_MINUS time  */
#line 1043 "gram.y"
                                 { global.time.sec = -global.time.sec; }
#line 2458 "gram.tab.c"
    break;

  case 35: /* time: number TOK_COLON number TOK_COLON float  */
#line 1048 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].dval) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%.6f) in time clause more than 59", (yyvsp[0].dval));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-4].ival) * 3600 + (yyvsp[-2].ival) * 60 + (int)(yyvsp[0].dval);
			global.time.nsec = (int)(1000000000*(((yyvsp[0].dval) - (int)(yyvsp[0].dval)))+0.5);
		    }
#line 2475 "gram.tab.c"
    break;

  case 36: /* time: number TOK_COLON number TOK_COLON number  */
#line 1061 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%d) in time clause more than 59", (yyvsp[0].ival));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-4].ival) * 3600 + (yyvsp[-2].ival) * 60 + (yyvsp[0].ival);
		    }
#line 2491 "gram.tab.c"
    break;

  case 37: /* time: number TOK_COLON float  */
#line 1073 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].dval) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%.6f) in time clause more than 59", (yyvsp[0].dval));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-2].ival) * 60 + (int)(yyvsp[0].dval);
			global.time.nsec = (int)(1000000000*(((yyvsp[0].dval) - (int)(yyvsp[0].dval)))+0.5);
		    }
#line 2508 "gram.tab.c"
    break;

  case 38: /* time: number TOK_COLON number  */
#line 1086 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%d) in time clause more than 59", (yyvsp[0].ival));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-2].ival) * 60 + (yyvsp[0].ival);
		    }
#line 2524 "gram.tab.c"
    break;

  case 39: /* time: float  */
#line 1098 "gram.y"
                    {
			if ((yyvsp[0].dval) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%.6f) in time clause more than 59", (yyvsp[0].dval));
			    yywarn(mess);
			}
			global.time.sec = (int)(yyvsp[0].dval);
			global.time.nsec = (int)(1000000000*(((yyvsp[0].dval) - (int)(yyvsp[0].dval)))+0.5);
		    }
#line 2537 "gram.tab.c"
    break;

  case 40: /* time: number  */
#line 1107 "gram.y"
                    {
			if ((yyvsp[0].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%d) in time clause more than 59", (yyvsp[0].ival));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[0].ival);
			global.time.nsec = 0;
		    }
#line 2550 "gram.tab.c"
    break;

  case 41: /* $@1: %empty  */
#line 1118 "gram.y"
                    {
			if (current_star_indom) {
			    __pmContext		*ctxp;
			    __pmHashCtl		*hcp;
			    __pmHashNode	*node;

			    ctxp = __pmHandleToPtr(pmWhichContext());
			    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			    PM_UNLOCK(ctxp->c_lock);
			    hcp = &ctxp->c_archctl->ac_log->hashindom;
			    star_domain = pmInDom_domain((yyvsp[0].indom));
			    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
				 node != NULL;
				 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
				if (pmInDom_domain((pmInDom)(node->key)) == star_domain)
				    current_indomspec = start_indom((pmInDom)(node->key));
			    }
			    do_walk_indom = 1;
			}
			else {
			    current_indomspec = start_indom((yyvsp[0].indom));
			    do_walk_indom = 0;
			}
		    }
#line 2586 "gram.tab.c"
    break;

  case 43: /* indomspec: TOK_INDOM  */
#line 1151 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<serial> or <domain>.* in indom rule");
			yyerror(mess);
		    }
#line 2595 "gram.tab.c"
    break;

  case 44: /* indom_int: TOK_FLOAT  */
#line 1158 "gram.y"
                    {
			int		domain;
			int		serial;
			int		sts;
			sts = sscanf((yyvsp[0].str), "%d.%d", &domain, &serial);
			if (sts < 2) {
			    pmsprintf(mess, sizeof(mess), "Missing serial field for indom");
			    yyerror(mess);
			}
			if (domain < 1 || domain >= DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for indom", domain);
			    yyerror(mess);
			}
			if (serial < 0 || serial >= 4194304) {
			    pmsprintf(mess, sizeof(mess), "Illegal serial field (%d) for indom", serial);
			    yyerror(mess);
			}
			current_star_indom = 0;
			free((yyvsp[0].str));
			(yyval.indom) = pmInDom_build(domain, serial);
		    }
#line 2621 "gram.tab.c"
    break;

  case 45: /* indom_int: TOK_INDOM_STAR  */
#line 1180 "gram.y"
                    {
			int		domain;
			sscanf((yyvsp[0].str), "%d.", &domain);
			if (domain < 1 || domain >= DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for indom", domain);
			    yyerror(mess);
			}
			current_star_indom = 1;
			free((yyvsp[0].str));
			(yyval.indom) = pmInDom_build(domain, 0);
		    }
#line 2637 "gram.tab.c"
    break;

  case 50: /* indomopt: TOK_INDOM TOK_ASSIGN duplicateopt indom_int  */
#line 1202 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    pmInDom	indom;
			    if (indom_root->new_indom != indom_root->old_indom) {
				pmsprintf(mess, sizeof(mess), "Duplicate indom clause for indom %s", pmInDomStr(indom_root->old_indom));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(ip->old_indom));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != ip->old_indom)
				ip->new_indom = indom;
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Instance domain %s: indom: No change", pmInDomStr(ip->old_indom));
				    yywarn(mess);
				}
			    }
			    ip->indom_flags |= (yyvsp[-1].ival);
			}
		    }
#line 2666 "gram.tab.c"
    break;

  case 51: /* indomopt: TOK_INAME TOK_STRING TOK_ASSIGN TOK_STRING  */
#line 1227 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_name(ip->old_indom, (yyvsp[-2].str), (yyvsp[0].str)) < 0)
				yyerror(mess);
			}
			free((yyvsp[-2].str));
			/* Note: $4 referenced from new_iname[] */
		    }
#line 2680 "gram.tab.c"
    break;

  case 52: /* indomopt: TOK_INAME TOK_STRING TOK_ASSIGN TOK_DELETE  */
#line 1237 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_name(ip->old_indom, (yyvsp[-2].str), NULL) < 0)
			    	yyerror(mess);
			}
			free((yyvsp[-2].str));
		    }
#line 2693 "gram.tab.c"
    break;

  case 53: /* indomopt: TOK_INST number TOK_ASSIGN number  */
#line 1246 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_inst(ip->old_indom, (yyvsp[-2].ival), (yyvsp[0].ival)) < 0)
				yyerror(mess);
			}
		    }
#line 2705 "gram.tab.c"
    break;

  case 54: /* indomopt: TOK_INST number TOK_ASSIGN TOK_DELETE  */
#line 1254 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_inst(ip->old_indom, (yyvsp[-2].ival), PM_IN_NULL) < 0)
				yyerror(mess);
			}
		    }
#line 2717 "gram.tab.c"
    break;

  case 55: /* indomopt: TOK_INDOM TOK_ASSIGN  */
#line 1262 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<serial> or <domain>.* in indom clause");
			yyerror(mess);
		    }
#line 2726 "gram.tab.c"
    break;

  case 56: /* indomopt: TOK_INDOM  */
#line 1267 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in indom clause");
			yyerror(mess);
		    }
#line 2735 "gram.tab.c"
    break;

  case 57: /* indomopt: TOK_INAME TOK_STRING TOK_ASSIGN  */
#line 1272 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting new external instance name string or DELETE in iname clause");
			yyerror(mess);
		    }
#line 2744 "gram.tab.c"
    break;

  case 58: /* indomopt: TOK_INAME TOK_STRING  */
#line 1277 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in iname clause");
			yyerror(mess);
		    }
#line 2753 "gram.tab.c"
    break;

  case 59: /* indomopt: TOK_INAME  */
#line 1282 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting old external instance name string in iname clause");
			yyerror(mess);
		    }
#line 2762 "gram.tab.c"
    break;

  case 60: /* indomopt: TOK_INST number TOK_ASSIGN  */
#line 1287 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting new internal instance identifier or DELETE in inst clause");
			yyerror(mess);
		    }
#line 2771 "gram.tab.c"
    break;

  case 61: /* indomopt: TOK_INST number  */
#line 1292 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in inst clause");
			yyerror(mess);
		    }
#line 2780 "gram.tab.c"
    break;

  case 62: /* indomopt: TOK_INST  */
#line 1297 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting old internal instance identifier in inst clause");
			yyerror(mess);
		    }
#line 2789 "gram.tab.c"
    break;

  case 63: /* duplicateopt: TOK_DUPLICATE  */
#line 1303 "gram.y"
                                        { (yyval.ival) = INDOM_DUPLICATE; }
#line 2795 "gram.tab.c"
    break;

  case 64: /* duplicateopt: %empty  */
#line 1304 "gram.y"
                                        { (yyval.ival) = 0; }
#line 2801 "gram.tab.c"
    break;

  case 65: /* $@2: %empty  */
#line 1308 "gram.y"
                    {
			if (current_star_metric) {
			    __pmContext		*ctxp;
			    __pmHashCtl		*hcp;
			    __pmHashNode	*node;

			    ctxp = __pmHandleToPtr(pmWhichContext());
			    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			    PM_UNLOCK(ctxp->c_lock);
			    hcp = &ctxp->c_archctl->ac_log->hashpmid;
			    star_domain = pmID_domain((yyvsp[0].pmid));
			    if (current_star_metric == 1)
				star_cluster = pmID_cluster((yyvsp[0].pmid));
			    else
				star_cluster = PM_ID_NULL;
			    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
				 node != NULL;
				 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
				if (pmID_domain((pmID)(node->key)) == star_domain &&
				    (star_cluster == PM_ID_NULL ||
				     star_cluster == pmID_cluster((pmID)(node->key))))
				    current_metricspec = start_metric((pmID)(node->key));
			    }
			    do_walk_metric = 1;
			}
			else {
			    if ((yyvsp[0].pmid) == PM_ID_NULL)
				/* metric not in archive */
				current_metricspec = NULL;
			    else
				current_metricspec = start_metric((yyvsp[0].pmid));
			    do_walk_metric = 0;
			}
		    }
#line 2847 "gram.tab.c"
    break;

  case 67: /* metricspec: TOK_METRIC  */
#line 1351 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting metric name or <domain>.<cluster>.<item> or <domain>.<cluster>.* or <domain>.*.* in metric rule");
			yyerror(mess);
		    }
#line 2856 "gram.tab.c"
    break;

  case 69: /* pmid_or_name: TOK_GNAME  */
#line 1359 "gram.y"
                    {
			int	sts;
			pmID	pmid;
			sts = pmLookupName(1, (const char **)&(yyvsp[0].str), &pmid);
			if (sts < 0) {
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Metric: %s: %s", (yyvsp[0].str), pmErrStr(sts));
				yywarn(mess);
			    }
			    pmid = PM_ID_NULL;
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.pmid) = pmid;
		    }
#line 2876 "gram.tab.c"
    break;

  case 70: /* pmid_int: TOK_PMID_INT  */
#line 1377 "gram.y"
                    {
			int	domain;
			int	cluster;
			int	item;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.%d.%d", &domain, &cluster, &item);
			assert(sts == 3);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			if (cluster < 0 || cluster >= 4096) {
			    pmsprintf(mess, sizeof(mess), "Illegal cluster field (%d) for pmid", cluster);
			    yyerror(mess);
			}
			if (item < 0 || item >= 1024) {
			    pmsprintf(mess, sizeof(mess), "Illegal item field (%d) for pmid", item);
			    yyerror(mess);
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.pmid) = pmID_build(domain, cluster, item);
		    }
#line 2908 "gram.tab.c"
    break;

  case 71: /* pmid_int: TOK_PMID_STAR  */
#line 1405 "gram.y"
                    {
			int	domain;
			int	cluster;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.%d.", &domain, &cluster);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			if (sts == 2) {
			    if (cluster >= 4096) {
				pmsprintf(mess, sizeof(mess), "Illegal cluster field (%d) for pmid", cluster);
				yyerror(mess);
			    }
			    current_star_metric = 1;
			}
			else {
			    cluster = 0;
			    current_star_metric = 2;
			}
			free((yyvsp[0].str));
			(yyval.pmid) = pmID_build(domain, cluster, 0);
		    }
#line 2940 "gram.tab.c"
    break;

  case 76: /* metricopt: TOK_PMID TOK_ASSIGN pmid_int  */
#line 1443 "gram.y"
                    {
			metricspec_t	*mp;
			pmID		pmid;
			for (mp = walk_metric(W_START, METRIC_CHANGE_PMID, "pmid", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_PMID, "pmid", 0)) {
			    if (current_star_metric == 1)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster((yyvsp[0].pmid)), pmID_item(mp->old_desc.pmid));
			    else if (current_star_metric == 2)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster(mp->old_desc.pmid), pmID_item(mp->old_desc.pmid));
			    else
				pmid = (yyvsp[0].pmid);
			    if (pmid == mp->old_desc.pmid) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): pmid: No change", mp->old_name, pmIDStr(mp->old_desc.pmid));
				    yywarn(mess);
				}
			    }
			    else {
				mp->new_desc.pmid = pmid;
				mp->flags |= METRIC_CHANGE_PMID;
			    }
			}
		    }
#line 2968 "gram.tab.c"
    break;

  case 77: /* metricopt: TOK_NAME TOK_ASSIGN TOK_GNAME  */
#line 1467 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_NAME, "name", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_NAME, "name", 0)) {
			    if (strcmp((yyvsp[0].str), mp->old_name) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): name: No change", mp->old_name, pmIDStr(mp->old_desc.pmid));
				    yywarn(mess);
				}
			    }
			    else {
				int	sts;
				pmID	pmid;
				sts = pmLookupName(1, (const char **)&(yyvsp[0].str), &pmid);
				if (sts >= 0) {
				    pmsprintf(mess, sizeof(mess), "Metric name %s already assigned for PMID %s", (yyvsp[0].str), pmIDStr(pmid));
				    yyerror(mess);
				}
				mp->new_name = (yyvsp[0].str);
				mp->flags |= METRIC_CHANGE_NAME;
			    }
			}
		    }
#line 2996 "gram.tab.c"
    break;

  case 78: /* metricopt: TOK_TYPE TOK_ASSIGN TOK_TYPE_NAME  */
#line 1491 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_TYPE, "type", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_TYPE, "type", 0)) {
			    if ((yyvsp[0].ival) == mp->old_desc.type) {
				/* old == new, so no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): type: PM_TYPE_%s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), pmTypeStr(mp->old_desc.type));
				    yywarn(mess);
				}
			    }
			    else {
				if (mp->old_desc.type == PM_TYPE_32 ||
				    mp->old_desc.type == PM_TYPE_U32 ||
				    mp->old_desc.type == PM_TYPE_64 ||
				    mp->old_desc.type == PM_TYPE_U64 ||
				    mp->old_desc.type == PM_TYPE_FLOAT ||
				    mp->old_desc.type == PM_TYPE_DOUBLE) {
				    mp->new_desc.type = (yyvsp[0].ival);
				    mp->flags |= METRIC_CHANGE_TYPE;
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Old type (PM_TYPE_%s) must be numeric", pmTypeStr(mp->old_desc.type));
				    yyerror(mess);
				}
			    }
			}
		    }
#line 3028 "gram.tab.c"
    break;

  case 79: /* metricopt: TOK_TYPE TOK_IF TOK_TYPE_NAME TOK_ASSIGN TOK_TYPE_NAME  */
#line 1519 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_TYPE, "type", 1); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_TYPE, "type", 1)) {
			    if (mp->old_desc.type != (yyvsp[-2].ival)) {
				if (wflag) {
				    char	tbuf0[20];
				    char	tbuf1[20];
				    pmTypeStr_r(mp->old_desc.type, tbuf0, sizeof(tbuf0));
				    pmTypeStr_r((yyvsp[0].ival), tbuf1, sizeof(tbuf1));
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): type: PM_TYPE_%s: No conditional change to PM_TYPE_%s", mp->old_name, pmIDStr(mp->old_desc.pmid), tbuf0, tbuf1);
				    yywarn(mess);
				}
			    }
			    else {
				if (mp->old_desc.type == PM_TYPE_32 ||
				    mp->old_desc.type == PM_TYPE_U32 ||
				    mp->old_desc.type == PM_TYPE_64 ||
				    mp->old_desc.type == PM_TYPE_U64 ||
				    mp->old_desc.type == PM_TYPE_FLOAT ||
				    mp->old_desc.type == PM_TYPE_DOUBLE) {
				    mp->new_desc.type = (yyvsp[0].ival);
				    mp->flags |= METRIC_CHANGE_TYPE;
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Old type (PM_TYPE_%s) must be numeric", pmTypeStr(mp->old_desc.type));
				    yyerror(mess);
				}
			    }
			}
		    }
#line 3063 "gram.tab.c"
    break;

  case 80: /* metricopt: TOK_INDOM TOK_ASSIGN null_or_indom pick  */
#line 1550 "gram.y"
                    {
			metricspec_t	*mp;
			pmInDom		indom;
			for (mp = walk_metric(W_START, METRIC_CHANGE_INDOM, "indom", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_INDOM, "indom", 0)) {
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[-1].indom)), pmInDom_serial(mp->old_desc.indom));
			    else
				indom = (yyvsp[-1].indom);
			    if (indom == mp->old_desc.indom) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): indom: %s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), pmInDomStr(mp->old_desc.indom));
				    yywarn(mess);
				}
			    }
			    else {
				if ((output == OUTPUT_MIN ||
					  output == OUTPUT_MAX ||
					  output == OUTPUT_SUM ||
					  output == OUTPUT_AVG) &&
					 mp->old_desc.type != PM_TYPE_32 &&
					 mp->old_desc.type != PM_TYPE_U32 &&
					 mp->old_desc.type != PM_TYPE_64 &&
					 mp->old_desc.type != PM_TYPE_U64 &&
					 mp->old_desc.type != PM_TYPE_FLOAT &&
					 mp->old_desc.type != PM_TYPE_DOUBLE) {
				    pmsprintf(mess, sizeof(mess), "OUTPUT option MIN, MAX, AVG or SUM requires type to be numeric, not PM_TYPE_%s", pmTypeStr(mp->old_desc.type));
				    yyerror(mess);
				}
				mp->new_desc.indom = indom;
				mp->flags |= METRIC_CHANGE_INDOM;
				mp->output = output;
				if (output == OUTPUT_ONE) {
				    mp->one_name = one_name;
				    mp->one_inst = one_inst;
				    if (mp->old_desc.indom == PM_INDOM_NULL)
					/*
					 * singular input, pick first (only)
					 * value, not one_inst matching ...
					 * one_inst used for output instance
					 * id
					 */
					mp->output = OUTPUT_FIRST;
				}
				if (output == OUTPUT_ALL) {
				    /*
				     * No OUTPUT clause, set up the defaults
				     * based on indom types:
				     * non-NULL -> NULL
				     *		OUTPUT_FIRST, inst PM_IN_NULL
				     * NULL -> non-NULL
				     *		OUTPUT_FIRST, inst 0
				     * non-NULL -> non-NULL
				     * 		all instances selected
				     *		(nothing to do for defaults)
				     * NULL -> NULL
				     *		caught above in no change case
				     */
				    if (mp->old_desc.indom != PM_INDOM_NULL &&
				        mp->new_desc.indom == PM_INDOM_NULL) {
					mp->output = OUTPUT_FIRST;
					mp->one_inst = PM_IN_NULL;
				    }
				    else if (mp->old_desc.indom == PM_INDOM_NULL &&
				             mp->new_desc.indom != PM_INDOM_NULL) {
					mp->output = OUTPUT_FIRST;
					mp->one_inst = 0;
				    }
				}
			    }
			}
			output = OUTPUT_ALL;	/* for next time */
		    }
#line 3141 "gram.tab.c"
    break;

  case 81: /* metricopt: TOK_SEM TOK_ASSIGN TOK_SEM_NAME  */
#line 1624 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_SEM, "sem", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_SEM, "sem", 0)) {
			    if ((yyvsp[0].ival) == mp->old_desc.sem) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): sem: %s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), SemStr(mp->old_desc.sem));
				    yywarn(mess);
				}
			    }
			    else {
				mp->new_desc.sem = (yyvsp[0].ival);
				mp->flags |= METRIC_CHANGE_SEM;
			    }
			}
		    }
#line 3162 "gram.tab.c"
    break;

  case 82: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA TOK_SPACE_NAME TOK_COMMA TOK_TIME_NAME TOK_COMMA TOK_COUNT_NAME rescaleopt  */
#line 1641 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_UNITS, "units", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_UNITS, "units", 0)) {
			    if ((yyvsp[-11].ival) == mp->old_desc.units.dimSpace &&
			        (yyvsp[-9].ival) == mp->old_desc.units.dimTime &&
			        (yyvsp[-7].ival) == mp->old_desc.units.dimCount &&
			        (yyvsp[-5].ival) == mp->old_desc.units.scaleSpace &&
			        (yyvsp[-3].ival) == mp->old_desc.units.scaleTime &&
			        (yyvsp[-1].ival) == mp->old_desc.units.scaleCount) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): units: %s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), pmUnitsStr(&mp->old_desc.units));
				    yywarn(mess);
				}
			    }
			    else {
				mp->new_desc.units.dimSpace = (yyvsp[-11].ival);
				mp->new_desc.units.dimTime = (yyvsp[-9].ival);
				mp->new_desc.units.dimCount = (yyvsp[-7].ival);
				mp->new_desc.units.scaleSpace = (yyvsp[-5].ival);
				mp->new_desc.units.scaleTime = (yyvsp[-3].ival);
				mp->new_desc.units.scaleCount = (yyvsp[-1].ival);
				mp->flags |= METRIC_CHANGE_UNITS;
				if ((yyvsp[0].ival) == 1) {
				    if ((yyvsp[-11].ival) == mp->old_desc.units.dimSpace &&
					(yyvsp[-9].ival) == mp->old_desc.units.dimTime &&
					(yyvsp[-7].ival) == mp->old_desc.units.dimCount)
					/* OK, no dim changes */
					mp->flags |= METRIC_RESCALE;
				    else {
					if (wflag) {
					    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): Dimension changed, cannot rescale", mp->old_name, pmIDStr(mp->old_desc.pmid));
					    yywarn(mess);
					}
				    }
				}
				else if (sflag) {
				    if ((yyvsp[-11].ival) == mp->old_desc.units.dimSpace &&
					(yyvsp[-9].ival) == mp->old_desc.units.dimTime &&
					(yyvsp[-7].ival) == mp->old_desc.units.dimCount)
					mp->flags |= METRIC_RESCALE;
				}
			    }
			}
		    }
#line 3212 "gram.tab.c"
    break;

  case 83: /* metricopt: TOK_DELETE  */
#line 1687 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_DELETE, "delete", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_DELETE, "delete", 0)) {
			    mp->flags |= METRIC_DELETE;
			}
		    }
#line 3223 "gram.tab.c"
    break;

  case 84: /* metricopt: TOK_PMID TOK_ASSIGN  */
#line 1694 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<cluster>.<item> or <domain>.<cluster>.* or <domain>.*.* in pmid clause");
			yyerror(mess);
		    }
#line 3232 "gram.tab.c"
    break;

  case 85: /* metricopt: TOK_NAME TOK_ASSIGN  */
#line 1699 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting metric name in iname clause");
			yyerror(mess);
		    }
#line 3241 "gram.tab.c"
    break;

  case 86: /* metricopt: TOK_TYPE TOK_ASSIGN  */
#line 1704 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_TYPE_XXX) in type clause");
			yyerror(mess);
		    }
#line 3250 "gram.tab.c"
    break;

  case 87: /* metricopt: TOK_TYPE TOK_IF  */
#line 1709 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_TYPE_XXX) after if in type clause");
			yyerror(mess);
		    }
#line 3259 "gram.tab.c"
    break;

  case 88: /* metricopt: TOK_TYPE TOK_IF TOK_TYPE_NAME TOK_ASSIGN  */
#line 1714 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_TYPE_XXX) in type clause");
			yyerror(mess);
		    }
#line 3268 "gram.tab.c"
    break;

  case 89: /* metricopt: TOK_INDOM TOK_ASSIGN  */
#line 1719 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<serial> or NULL in indom clause");
			yyerror(mess);
		    }
#line 3277 "gram.tab.c"
    break;

  case 90: /* metricopt: TOK_SEM TOK_ASSIGN  */
#line 1724 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_SEM_XXX) in sem clause");
			yyerror(mess);
		    }
#line 3286 "gram.tab.c"
    break;

  case 91: /* metricopt: TOK_UNITS TOK_ASSIGN  */
#line 1729 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 3 numeric values for dim* fields of units");
			yyerror(mess);
		    }
#line 3295 "gram.tab.c"
    break;

  case 92: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA  */
#line 1734 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 0 or XXX (from PM_SPACE_XXX) for scaleSpace field of units");
			yyerror(mess);
		    }
#line 3304 "gram.tab.c"
    break;

  case 93: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA TOK_SPACE_NAME TOK_COMMA  */
#line 1739 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 0 or XXX (from PM_TIME_XXX) for scaleTime field of units");
			yyerror(mess);
		    }
#line 3313 "gram.tab.c"
    break;

  case 94: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA TOK_SPACE_NAME TOK_COMMA TOK_TIME_NAME TOK_COMMA  */
#line 1744 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 0 or ONE for scaleCount field of units");
			yyerror(mess);
		    }
#line 3322 "gram.tab.c"
    break;

  case 96: /* null_or_indom: TOK_NULL_INT  */
#line 1752 "gram.y"
                    {
			(yyval.indom) = PM_INDOM_NULL;
		    }
#line 3330 "gram.tab.c"
    break;

  case 97: /* pick: TOK_OUTPUT TOK_INST number  */
#line 1758 "gram.y"
                    {
			output = OUTPUT_ONE;
			one_inst = (yyvsp[0].ival);
			one_name = NULL;
		    }
#line 3340 "gram.tab.c"
    break;

  case 98: /* pick: TOK_OUTPUT TOK_INAME TOK_STRING  */
#line 1764 "gram.y"
                    {
			output = OUTPUT_ONE;
			one_inst = PM_IN_NULL;
			one_name = (yyvsp[0].str);
		    }
#line 3350 "gram.tab.c"
    break;

  case 99: /* pick: TOK_OUTPUT TOK_OUTPUT_TYPE  */
#line 1770 "gram.y"
                    {
			output = (yyvsp[0].ival);
		    }
#line 3358 "gram.tab.c"
    break;

  case 100: /* pick: TOK_OUTPUT  */
#line 1774 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting FIRST or LAST or INST or INAME or MIN or MAX or AVG for OUTPUT instance option");
			yyerror(mess);
		    }
#line 3367 "gram.tab.c"
    break;

  case 102: /* rescaleopt: TOK_RESCALE  */
#line 1781 "gram.y"
                              { (yyval.ival) = 1; }
#line 3373 "gram.tab.c"
    break;

  case 103: /* rescaleopt: %empty  */
#line 1783 "gram.y"
                    { (yyval.ival) = 0; }
#line 3379 "gram.tab.c"
    break;

  case 107: /* $@3: %empty  */
#line 1794 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		target_types;
			int		this_type;
			int		type;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_textspec = NULL;
			if ((yyvsp[-2].pmid) == PM_ID_NULL) {
			    /* Metric referenced by name is not in the archive */
			    do_walk_text = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_metric) {
				/* Set up for metrics specified using globbing. */
				star_domain = pmID_domain((yyvsp[-2].pmid));
				if (current_star_metric == 1)
				    star_cluster = pmID_cluster((yyvsp[-2].pmid));
				else
				    star_cluster = PM_ID_NULL;
			    }

			    /* We're looking for text of the specified type(s) for metric(s). */
			    target_types = PM_TEXT_PMID | (yyvsp[-1].ival);
			    hcp1 = &ctxp->c_archctl->ac_log->hashtext;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if ((this_type & target_types) != this_type)
				    continue;

				/*
				 * Collect the text records associated with the specified
				 * metric(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_metric) {
					/* Match the globbed metric spec and keep looking. */
					if (pmID_domain((pmID)(node2->key)) == star_domain &&
					    (star_cluster == PM_ID_NULL ||
					     star_cluster == pmID_cluster((pmID)(node2->key)))) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }

					}
				    }
				    else {
					/* Match the exact metric PMID. */
					if ((pmID)(node2->key) == (yyvsp[-2].pmid)) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }
					    /*
					     * Stop looking if we have found all of the specified
					     * types.
					     */
					    type = this_type & PM_TEXT_TYPE_MASK;
					    target_types ^= type;
					    if ((target_types & PM_TEXT_TYPE_MASK) == 0)
						break;
					}
				    }
				}
				if ((target_types & PM_TEXT_TYPE_MASK) == 0)
				    break;
			    }
			    do_walk_text = (found > 1);
			}
		    }
#line 3480 "gram.tab.c"
    break;

  case 109: /* opttexttypes: texttypes  */
#line 1894 "gram.y"
                    { (yyval.ival) = (yyvsp[0].ival); }
#line 3486 "gram.tab.c"
    break;

  case 110: /* opttexttypes: %empty  */
#line 1896 "gram.y"
                    { (yyval.ival) = PM_TEXT_ONELINE; }
#line 3492 "gram.tab.c"
    break;

  case 111: /* texttypes: texttype  */
#line 1900 "gram.y"
                    { (yyval.ival) = (yyvsp[0].ival); }
#line 3498 "gram.tab.c"
    break;

  case 112: /* texttypes: texttype texttypes  */
#line 1902 "gram.y"
                    { (yyval.ival) = (yyvsp[-1].ival) | (yyvsp[0].ival); }
#line 3504 "gram.tab.c"
    break;

  case 113: /* texttype: TOK_TEXT_STAR  */
#line 1906 "gram.y"
                    { (yyval.ival) = PM_TEXT_ONELINE | PM_TEXT_HELP; }
#line 3510 "gram.tab.c"
    break;

  case 114: /* texttype: TOK_HELP  */
#line 1908 "gram.y"
                    { (yyval.ival) = PM_TEXT_HELP; }
#line 3516 "gram.tab.c"
    break;

  case 115: /* texttype: TOK_ONELINE  */
#line 1910 "gram.y"
                    { (yyval.ival) = PM_TEXT_ONELINE; }
#line 3522 "gram.tab.c"
    break;

  case 116: /* opttextvalue: textstring  */
#line 1914 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3528 "gram.tab.c"
    break;

  case 117: /* opttextvalue: %empty  */
#line 1916 "gram.y"
                    { (yyval.str) = NULL; }
#line 3534 "gram.tab.c"
    break;

  case 119: /* opttextmetricoptlist: %empty  */
#line 1921 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_ACTIVE, "active", 0); tp != NULL; tp = walk_text(W_NEXT, 0, "", 0)) {
			    tp->flags &= ~TEXT_ACTIVE;
			}
		    }
#line 3545 "gram.tab.c"
    break;

  case 122: /* textmetricopt: TOK_DELETE  */
#line 1934 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_DELETE, "delete", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_DELETE, "delete", 0)) {
			    tp->flags |= TEXT_DELETE;
			}
		    }
#line 3556 "gram.tab.c"
    break;

  case 123: /* textmetricopt: TOK_TEXT TOK_ASSIGN textstring  */
#line 1941 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_CHANGE_TEXT, "text", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_TEXT, "text", 0)) {
			    if (tp->new_text) {
				if (strcmp(tp->new_text, (yyvsp[0].str)) == 0) {
				    pmsprintf(mess, sizeof(mess), "Duplicate text change clause");
				    yyerror(mess);
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Conflicting text change clause");
				    yyerror(mess);
				}
				free((yyvsp[0].str));
			    }
			    else if (strcmp(tp->old_text, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Help text: No change");
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				tp->new_text = (yyvsp[0].str);
				tp->flags |= TEXT_CHANGE_TEXT;
			    }
			}
		    }
#line 3589 "gram.tab.c"
    break;

  case 124: /* textmetricopt: TOK_METRIC TOK_ASSIGN pmid_int  */
#line 1970 "gram.y"
                    {
			textspec_t	*tp;
			pmID		pmid;
			for (tp = walk_text(W_START, TEXT_CHANGE_ID, "id", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_ID, "id", 0)) {
			    if (current_star_metric == 1)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster((yyvsp[0].pmid)), pmID_item(tp->old_id));
			    else if (current_star_metric == 2)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster(tp->old_id), pmID_item(tp->old_id));
			    else
				pmid = (yyvsp[0].pmid);
			    if (pmid == tp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Text for metric: %s: metric: No change", pmIDStr(tp->old_id));
				    yywarn(mess);
				}
			    }
			    else {
				const textspec_t *tp1;
				/* Warn if the target metric already has text of this type. */
				/*
				 * Search all of the change specs for others targeting
				 * the same metric.
				 */
				for (tp1 = text_root; tp1 != NULL; tp1 = tp1->t_next) {
				    if (! (tp1->flags & TEXT_CHANGE_ID))
					continue; /* not an id change spec */
				    if (tp1->new_type != tp->new_type)
					continue; /* target is a different type */
				    if (tp1 == tp)
					continue; /* same spec */
				    if (tp1->new_id == pmid) {
					/* conflict with another change spec. */
					pmsprintf(mess, sizeof(mess), "Text for metric: %s: metric: Conflicting metric change", pmIDStr(tp->old_id));
					yyerror(mess);
					break;
				    }
				}
				if (tp1 == NULL) {
				    /* No conflict */
				    tp->new_id = pmid;
				    tp->flags |= TEXT_CHANGE_ID;
				}
			    }
			}
		    }
#line 3640 "gram.tab.c"
    break;

  case 125: /* textstring: TOK_STRING  */
#line 2019 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3646 "gram.tab.c"
    break;

  case 126: /* textstring: TOK_TEXT_STRING  */
#line 2021 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3652 "gram.tab.c"
    break;

  case 127: /* $@4: %empty  */
#line 2025 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		target_types;
			int		this_type;
			int		type;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_textspec = NULL;
			if ((yyvsp[-2].indom) == PM_ID_NULL) {
			    /* Indom is not in the archive */
			    do_walk_text = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_indom) {
				/* Set up for indoms specified using globbing. */
				star_domain = pmInDom_domain((yyvsp[-2].indom));
			    }

			    /* We're looking for text of the specified type(s) for indom(s). */
			    target_types = PM_TEXT_INDOM | (yyvsp[-1].ival);
			    hcp1 = &ctxp->c_archctl->ac_log->hashtext;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if ((this_type & target_types) != this_type)
				    continue;

				/*
				 * Collect the text records associated with the specified
				 * metric(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_indom) {
					/* Match the globbed metric spec and keep looking. */
					if (pmInDom_domain((pmID)(node2->key)) == star_domain) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }
					}
				    }
				    else {
					/* Match the exact indom id. */
					if ((pmID)(node2->key) == (yyvsp[-2].indom)) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }
					    /*
					     * Stop looking if we have found all of the specified
					     * types.
					     */
					    type = this_type & PM_TEXT_TYPE_MASK;
					    target_types ^= type;
					    if ((target_types & PM_TEXT_TYPE_MASK) == 0)
						break;
					}
				    }
				}
				if ((target_types & PM_TEXT_TYPE_MASK) == 0)
				    break;
			    }
			    do_walk_text = (found > 1);
			}
		    }
#line 3746 "gram.tab.c"
    break;

  case 130: /* opttextindomoptlist: %empty  */
#line 2119 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_ACTIVE, "active", 0); tp != NULL; tp = walk_text(W_NEXT, 0, "", 0)) {
			    tp->flags &= ~TEXT_ACTIVE;
			}
		    }
#line 3757 "gram.tab.c"
    break;

  case 133: /* textindomopt: TOK_DELETE  */
#line 2132 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_DELETE, "delete", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_DELETE, "delete", 0)) {
			    tp->flags |= TEXT_DELETE;
			}
		    }
#line 3768 "gram.tab.c"
    break;

  case 134: /* textindomopt: TOK_TEXT TOK_ASSIGN textstring  */
#line 2139 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_CHANGE_TEXT, "text", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_TEXT, "text", 0)) {
			    if (tp->new_text) {
				if (strcmp(tp->new_text, (yyvsp[0].str)) == 0) {
				    pmsprintf(mess, sizeof(mess), "Duplicate text change clause");
				    yyerror(mess);
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Conflicting text change clause");
				    yyerror(mess);
				}
				free((yyvsp[0].str));
			    }
			    else if (strcmp(tp->old_text, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Help text: No change");
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				tp->new_text = (yyvsp[0].str);
				tp->flags |= TEXT_CHANGE_TEXT;
			    }
			}
		    }
#line 3801 "gram.tab.c"
    break;

  case 135: /* textindomopt: TOK_INDOM TOK_ASSIGN indom_int  */
#line 2168 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_CHANGE_ID, "id", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_ID, "id", 0)) {
			    pmInDom	indom;
			    if (tp->new_id != tp->old_id) {
				pmsprintf(mess, sizeof(mess), "Duplicate text clause for indom %s", pmInDomStr(tp->old_id));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(tp->old_id));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != tp->old_id) {
				tp->new_id = indom;
				tp->flags |= TEXT_CHANGE_ID;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Text for instance domain %s: indom: No change", pmInDomStr(tp->old_id));
				    yywarn(mess);
				}
			    }
			}
		    }
#line 3831 "gram.tab.c"
    break;

  case 143: /* $@5: %empty  */
#line 2207 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		found = 0;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			/* We're looking for context labels. */
			current_labelspec = NULL;
			hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
			     node1 != NULL;
			     node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
			    __pmHashCtl	*hcp2;
			    __pmHashNode	*node2;
			    int		this_type;

			    /* Was this object type selected? */
			    this_type = (int)(node1->key);
			    if (this_type != PM_LABEL_CONTEXT)
				continue;

			    /*
			     * Collect the label records associated with the specified
			     * metric(s).
			     */
			    hcp2 = (__pmHashCtl *)(node1->data);
			    for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				 node2 != NULL;
				 node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				/* We want all of the context labels. */
				current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
								current_label_name, current_label_value);
				if (current_labelspec) {
				    current_labelspec->flags |= LABEL_ACTIVE;
				    ++found;
				}
			    }
			}
			do_walk_label = found;
		    }
#line 3887 "gram.tab.c"
    break;

  case 145: /* optlabeldetails: jsonname optlabelvalue  */
#line 2262 "gram.y"
                    { current_label_name = (yyvsp[-1].str); }
#line 3893 "gram.tab.c"
    break;

  case 146: /* optlabeldetails: TOK_LABEL_STAR optlabelvalue  */
#line 2264 "gram.y"
                    { current_label_name = NULL; }
#line 3899 "gram.tab.c"
    break;

  case 147: /* optlabeldetails: %empty  */
#line 2266 "gram.y"
                    {
			current_label_name = NULL;
			current_label_value = NULL;
		    }
#line 3908 "gram.tab.c"
    break;

  case 148: /* jsonname: TOK_JSON_STRING  */
#line 2273 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3914 "gram.tab.c"
    break;

  case 149: /* jsonname: TOK_STRING  */
#line 2275 "gram.y"
                    {
			(yyval.str) = add_quotes((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 3923 "gram.tab.c"
    break;

  case 150: /* optlabelvalue: jsonvalue  */
#line 2282 "gram.y"
                    { current_label_value = (yyvsp[0].str); }
#line 3929 "gram.tab.c"
    break;

  case 151: /* optlabelvalue: TOK_LABEL_STAR  */
#line 2284 "gram.y"
                    { current_label_value = NULL; }
#line 3935 "gram.tab.c"
    break;

  case 152: /* optlabelvalue: %empty  */
#line 2286 "gram.y"
                    { current_label_value = NULL; }
#line 3941 "gram.tab.c"
    break;

  case 153: /* jsonvalue: jsonname  */
#line 2290 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3947 "gram.tab.c"
    break;

  case 154: /* jsonvalue: jsonnumber  */
#line 2292 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3953 "gram.tab.c"
    break;

  case 155: /* jsonvalue: TOK_JSON_TRUE  */
#line 2294 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3959 "gram.tab.c"
    break;

  case 156: /* jsonvalue: TOK_JSON_FALSE  */
#line 2296 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3965 "gram.tab.c"
    break;

  case 157: /* jsonvalue: TOK_JSON_NULL  */
#line 2298 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3971 "gram.tab.c"
    break;

  case 158: /* jsonnumber: TOK_MINUS TOK_NUMBER  */
#line 2302 "gram.y"
                    {
			(yyval.str) = dupcat("-", (yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 3980 "gram.tab.c"
    break;

  case 159: /* jsonnumber: TOK_NUMBER  */
#line 2307 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3986 "gram.tab.c"
    break;

  case 160: /* jsonnumber: TOK_MINUS TOK_JSON_NUMBER  */
#line 2309 "gram.y"
                    {
			(yyval.str) = dupcat("-", (yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 3995 "gram.tab.c"
    break;

  case 161: /* jsonnumber: TOK_JSON_NUMBER  */
#line 2314 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4001 "gram.tab.c"
    break;

  case 163: /* optlabelcontextoptlist: %empty  */
#line 2319 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4009 "gram.tab.c"
    break;

  case 166: /* labelcontextopt: TOK_DELETE  */
#line 2329 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4020 "gram.tab.c"
    break;

  case 167: /* labelcontextopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 2336 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for context");
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for context: label: No change");
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4047 "gram.tab.c"
    break;

  case 168: /* labelcontextopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 2360 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: value: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4074 "gram.tab.c"
    break;

  case 169: /* labelcontextopt: newlabelspec  */
#line 2384 "gram.y"
                    {
			new_context_label();
		    }
#line 4082 "gram.tab.c"
    break;

  case 170: /* newlabelspec: TOK_NEW jsonname jsonvalue  */
#line 2389 "gram.y"
                    {
			/* The current label name and value must both NOT be specified. */
			if (current_label_name || current_label_value) {
			    pmsprintf(mess, sizeof(mess), "The target label name and value must both not be specified for a NEW label and will be ignored");
			    yywarn(mess);
			    if (current_label_name)
				free(current_label_name);
			    if (current_label_value)
				free(current_label_value);
			}
			current_label_name = (yyvsp[-1].str);
			current_label_value = (yyvsp[0].str);
		    }
#line 4100 "gram.tab.c"
    break;

  case 171: /* $@6: %empty  */
#line 2404 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		 found = 0;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			assert ((yyvsp[-1].ival) != PM_ID_NULL);
			current_label_id = (yyvsp[-1].ival);
			    
			/* We're looking for domain labels. */
			hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
			     node1 != NULL;
			     node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
			    __pmHashCtl	*hcp2;
			    __pmHashNode	*node2;
			    int		this_type;

			    /* Was this object type selected? */
			    this_type = (int)(node1->key);
			    if (this_type != PM_LABEL_DOMAIN)
				continue;

			    /*
			     * Collect the label records associated with the specified
			     * metric domain.
			     */
			    hcp2 = (__pmHashCtl *)(node1->data);
			    for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				 node2 != NULL;
				 node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				/* Match the exact metric domain. */
				if ((pmID)(node2->key) == (yyvsp[-1].ival)) {
				    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
								    current_label_name, current_label_value);
				    if (current_labelspec) {
					current_labelspec->flags |= LABEL_ACTIVE;
					++found;
				    }
				}
			    }
			}
			do_walk_label = found;
		    }
#line 4161 "gram.tab.c"
    break;

  case 173: /* pmid_domain: TOK_NUMBER  */
#line 2464 "gram.y"
                    {
			int	domain;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d", &domain);
			assert(sts == 1);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.ival) = domain;
		    }
#line 4183 "gram.tab.c"
    break;

  case 175: /* optlabeldomainoptlist: %empty  */
#line 2485 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4191 "gram.tab.c"
    break;

  case 178: /* labeldomainopt: TOK_DELETE  */
#line 2495 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4202 "gram.tab.c"
    break;

  case 179: /* labeldomainopt: TOK_DOMAIN TOK_ASSIGN TOK_NUMBER  */
#line 2502 "gram.y"
                    {
			labelspec_t	*lp;
			int		domain;
			int		sts;
			sts = sscanf((yyvsp[0].str), "%d", &domain);
			assert(sts == 1);
			free((yyvsp[0].str));
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    if (domain == lp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric domain: %d: No change", lp->old_id);
				    yywarn(mess);
				}
			    }
			    else {
				/*
				 * No need to check for conflicts. Multiple label set records for
				 * the same domain are ok.
				 */
				lp->new_id = domain;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			}
		    }
#line 4232 "gram.tab.c"
    break;

  case 180: /* labeldomainopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 2528 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for domain %d", lp->old_id);
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for domain %d: label: No change", lp->old_id);
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4259 "gram.tab.c"
    break;

  case 181: /* labeldomainopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 2552 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: value: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4286 "gram.tab.c"
    break;

  case 182: /* labeldomainopt: newlabelspec  */
#line 2576 "gram.y"
                    {
			new_domain_label(current_label_id);
		    }
#line 4294 "gram.tab.c"
    break;

  case 183: /* $@7: %empty  */
#line 2582 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		found = 0;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			assert ((yyvsp[-1].ival) != PM_ID_NULL);
			current_label_id = (yyvsp[-1].ival);
			    
			if (current_star_metric) {
			    /* Set up for metrics specified using globbing. */
			    star_domain = pmID_domain((yyvsp[-1].ival));
			    star_cluster = PM_ID_NULL;
			}

			/* We're looking for cluster labels. */
			hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
			     node1 != NULL;
			     node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
			    __pmHashCtl		*hcp2;
			    __pmHashNode	*node2;
			    int			this_type;

			    /* Was this object type selected? */
			    this_type = (int)(node1->key);
			    if (this_type != PM_LABEL_CLUSTER)
				continue;

			    /*
			     * Collect the label records associated with the specified
			     * metric(s).
			     */
			    hcp2 = (__pmHashCtl *)(node1->data);
			    for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				 node2 != NULL;
				 node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				if (current_star_metric) {
				    /* Match the globbed cluster spec and keep looking. */
				    if (pmID_domain((pmID)(node2->key)) == star_domain) {
					current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									current_label_name, current_label_value);
					if (current_labelspec) {
					    current_labelspec->flags |= LABEL_ACTIVE;
					    ++found;
					}
				    }
				}
				else {
				    /* Match the exact cluster spec. */
				    if ((pmID)(node2->key) == (yyvsp[-1].ival)) {
					current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									current_label_name, current_label_value);
					if (current_labelspec) {
					    current_labelspec->flags |= LABEL_ACTIVE;
					    ++found;
					}
				    }
				}
			    }
			}
			do_walk_label = found;
		    }
#line 4374 "gram.tab.c"
    break;

  case 185: /* pmid_cluster: TOK_FLOAT  */
#line 2661 "gram.y"
                    {
			int	domain;
			int	cluster;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.%d", &domain, &cluster);
			assert(sts == 2);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			if (cluster < 0 || cluster >= 4096) {
			    pmsprintf(mess, sizeof(mess), "Illegal cluster field (%d) for pmid", cluster);
			    yyerror(mess);
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.ival) = pmID_build(domain, cluster, 0);
		    }
#line 4401 "gram.tab.c"
    break;

  case 186: /* pmid_cluster: TOK_INDOM_STAR  */
#line 2684 "gram.y"
                    {
			int	domain;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.", &domain);
			assert (sts == 1);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			current_star_metric = 1;
			free((yyvsp[0].str));
			(yyval.ival) = pmID_build(domain, 0, 0);
		    }
#line 4423 "gram.tab.c"
    break;

  case 188: /* optlabelclusteroptlist: %empty  */
#line 2705 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4431 "gram.tab.c"
    break;

  case 191: /* labelclusteropt: TOK_DELETE  */
#line 2715 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4442 "gram.tab.c"
    break;

  case 192: /* labelclusteropt: TOK_CLUSTER TOK_ASSIGN pmid_cluster  */
#line 2722 "gram.y"
                    {
			labelspec_t	*lp;
			int		cluster;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    if (current_star_metric)
				cluster = pmID_build(pmID_domain((yyvsp[0].ival)), pmID_cluster(lp->old_id), PM_ID_NULL);
			    else
				cluster = pmID_build(pmID_domain((yyvsp[0].ival)), pmID_cluster((yyvsp[0].ival)), PM_ID_NULL);
			    if (cluster == lp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster: %d.%d: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
			    }
			    else {
				/*
				 * No need to check for conflicts. Multiple label set records for
				 * the same cluster are ok.
				 */
				lp->new_id = cluster;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			}
		    }
#line 4472 "gram.tab.c"
    break;

  case 193: /* labelclusteropt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 2748 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: label: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4499 "gram.tab.c"
    break;

  case 194: /* labelclusteropt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 2772 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: value: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4526 "gram.tab.c"
    break;

  case 195: /* labelclusteropt: newlabelspec  */
#line 2796 "gram.y"
                    {
			new_cluster_label(current_label_id);
		    }
#line 4534 "gram.tab.c"
    break;

  case 196: /* $@8: %empty  */
#line 2802 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			current_label_id = (yyvsp[-1].pmid);
			if ((yyvsp[-1].pmid) == PM_ID_NULL) {
			    /* Metric referenced by name is not in the archive */
			    do_walk_label = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_metric) {
				/* Set up for metrics specified using globbing. */
				star_domain = pmID_domain((yyvsp[-1].pmid));
				if (current_star_metric == 1)
				    star_cluster = pmID_cluster((yyvsp[-1].pmid));
				else
				    star_cluster = PM_ID_NULL;
			    }

			    /* We're looking for item labels. */
			    hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;
				int		this_type;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if (this_type != PM_LABEL_ITEM)
				    continue;

				/*
				 * Collect the label records associated with the specified
				 * metric(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_metric) {
					/* Match the globbed metric spec and keep looking. */
					if (pmID_domain((pmID)(node2->key)) == star_domain &&
					    (star_cluster == PM_ID_NULL ||
					     star_cluster == pmID_cluster((pmID)(node2->key)))) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }

					}
				    }
				    else {
					/* Match the exact metric PMID. */
					if ((pmID)(node2->key) == (yyvsp[-1].pmid)) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				}
			    }
			    do_walk_label = found;
			}
		    }
#line 4625 "gram.tab.c"
    break;

  case 199: /* optlabelitemoptlist: %empty  */
#line 2893 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4633 "gram.tab.c"
    break;

  case 202: /* labelitemopt: TOK_DELETE  */
#line 2903 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4644 "gram.tab.c"
    break;

  case 203: /* labelitemopt: TOK_ITEM TOK_ASSIGN pmid_int  */
#line 2910 "gram.y"
                    {
			labelspec_t	*lp;
			pmID		pmid;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    if (current_star_metric == 1)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster((yyvsp[0].pmid)), pmID_item(lp->old_id));
			    else if (current_star_metric == 2)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster(lp->old_id), pmID_item(lp->old_id));
			    else
				pmid = (yyvsp[0].pmid);
			    if (pmid == lp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric: %s: item: No change", pmIDStr(lp->old_id));
				    yywarn(mess);
				}
			    }
			    else {
				/*
				 * No need to check for conflicts. Multiple label set records for
				 * the same pmid are ok.
				 */
				lp->new_id = pmid;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			}
		    }
#line 4676 "gram.tab.c"
    break;

  case 204: /* labelitemopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 2938 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for metric %s", pmIDStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric %s: label: No change", pmIDStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4703 "gram.tab.c"
    break;

  case 205: /* labelitemopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 2962 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for metric %s", pmIDStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric %s: value: No change", pmIDStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4730 "gram.tab.c"
    break;

  case 206: /* labelitemopt: newlabelspec  */
#line 2986 "gram.y"
                    {
			new_item_label(current_label_id);
		    }
#line 4738 "gram.tab.c"
    break;

  case 207: /* $@9: %empty  */
#line 2992 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			current_label_id = (yyvsp[-1].indom);
			if ((yyvsp[-1].indom) == PM_ID_NULL) {
			    /* Indom is not in the archive */
			    do_walk_label = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_indom) {
				/* Set up for indoms specified using globbing. */
				star_domain = pmInDom_domain((yyvsp[-1].indom));
			    }

			    /* We're looking for label sets for indom(s). */
			    hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;
				int		this_type;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if (this_type != PM_LABEL_INDOM)
				    continue;

				/*
				 * Collect the label records associated with the specified
				 * imdom(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_indom) {
					/* Match the globbed indom spec and keep looking. */
					if (pmInDom_domain((pmID)(node2->key)) == star_domain) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				    else {
					/* Match the exact indom id. */
					if ((pmID)(node2->key) == (yyvsp[-1].indom)) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				}
			    }
			    do_walk_label = found;
			}
		    }
#line 4822 "gram.tab.c"
    break;

  case 210: /* optlabelindomoptlist: %empty  */
#line 3076 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4830 "gram.tab.c"
    break;

  case 213: /* labelindomopt: TOK_DELETE  */
#line 3086 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4841 "gram.tab.c"
    break;

  case 214: /* labelindomopt: TOK_INDOM TOK_ASSIGN indom_int  */
#line 3093 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    pmInDom	indom;
			    if (lp->new_id != lp->old_id) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(lp->old_id));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != lp->old_id) {
				lp->new_id = indom;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: indom: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
			    }
			}
		    }
#line 4871 "gram.tab.c"
    break;

  case 215: /* labelindomopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 3119 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: label: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4898 "gram.tab.c"
    break;

  case 216: /* labelindomopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 3143 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: value: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4925 "gram.tab.c"
    break;

  case 217: /* labelindomopt: newlabelspec  */
#line 3167 "gram.y"
                    {
			new_indom_label(current_label_id);
		    }
#line 4933 "gram.tab.c"
    break;

  case 218: /* $@10: %empty  */
#line 3173 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			current_label_id = (yyvsp[-1].indom);
			if ((yyvsp[-1].indom) == PM_ID_NULL) {
			    /* Indom is not in the archive */
			    do_walk_label = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_indom) {
				/* Set up for indoms specified using globbing. */
				star_domain = pmInDom_domain((yyvsp[-1].indom));
			    }

			    /* We're looking for label sets for the instances of the indom(s). */
			    hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;
				int		this_type;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if (this_type != PM_LABEL_INSTANCES)
				    continue;

				/*
				 * Collect the label records associated with the specified
				 * indom instance(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_indom) {
					/* Match the globbed indom spec and keep looking. */
					if (pmInDom_domain((pmID)(node2->key)) == star_domain) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key),
									    current_label_instance, current_label_instance_name,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				    else {
					/* Match the exact indom id. */
					if ((pmID)(node2->key) == (yyvsp[-1].indom)) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key),
									    current_label_instance, current_label_instance_name,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				}
			    }
			    do_walk_label = found;
			}
		    }
#line 5019 "gram.tab.c"
    break;

  case 220: /* optinstancelabeldetails: TOK_STRING optlabeldetails  */
#line 3258 "gram.y"
                    {
			current_label_instance = -1;
			current_label_instance_name = (yyvsp[-1].str);
		    }
#line 5028 "gram.tab.c"
    break;

  case 221: /* optinstancelabeldetails: TOK_NUMBER optlabeldetails  */
#line 3263 "gram.y"
                    {
			current_label_instance = atoi((yyvsp[-1].str));
			free((yyvsp[-1].str));
			current_label_instance_name = NULL;
		    }
#line 5038 "gram.tab.c"
    break;

  case 222: /* optinstancelabeldetails: TOK_LABEL_STAR optlabeldetails  */
#line 3269 "gram.y"
                    {
			current_label_instance = -1;
			current_label_instance_name = NULL;
		    }
#line 5047 "gram.tab.c"
    break;

  case 223: /* optinstancelabeldetails: %empty  */
#line 3274 "gram.y"
                    {
			current_label_instance = -1;
			current_label_instance_name = NULL;
			current_label_name = NULL;
			current_label_value = NULL;
		    }
#line 5058 "gram.tab.c"
    break;

  case 225: /* optlabelinstancesoptlist: %empty  */
#line 3284 "gram.y"
                    {
			deactivate_labels();
		    }
#line 5066 "gram.tab.c"
    break;

  case 228: /* labelinstancesopt: TOK_DELETE  */
#line 3294 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 5077 "gram.tab.c"
    break;

  case 229: /* labelinstancesopt: TOK_INSTANCES TOK_ASSIGN indom_int  */
#line 3301 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    pmInDom	indom;
			    if (lp->new_id != lp->old_id) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(lp->old_id));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != lp->old_id) {
				lp->new_id = indom;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: indom: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
			    }
			}
		    }
#line 5107 "gram.tab.c"
    break;

  case 230: /* labelinstancesopt: TOK_INSTANCE TOK_ASSIGN TOK_NUMBER  */
#line 3327 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    int	instance;
			    if (lp->new_instance != lp->old_instance) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    instance = atoi((yyvsp[0].str));
			    free((yyvsp[0].str));
			    if (instance != lp->old_instance) {
				lp->new_instance = instance;
				lp->flags |= LABEL_CHANGE_INSTANCE;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s instance %d: instance: No change", pmInDomStr(lp->old_id), lp->old_instance);
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			}
		    }
#line 5136 "gram.tab.c"
    break;

  case 231: /* labelinstancesopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 3352 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instances of instance domain %s: label: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 5163 "gram.tab.c"
    break;

  case 232: /* labelinstancesopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 3376 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instances of instance domain %s: value: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 5190 "gram.tab.c"
    break;

  case 233: /* labelinstancesopt: newlabelspec  */
#line 3400 "gram.y"
                    {
			new_indom_instance_label(current_label_id);
		    }
#line 5198 "gram.tab.c"
    break;


#line 5202 "gram.tab.c"

      default: break;
    }
  /* User semantic actions sometimes alter yychar, and that requires
     that yytoken be updated with the new translation.  We take the
     approach of translating immediately before every use of yytoken.
     One alternative is translating here after every semantic action,
     but that translation would be missed if the semantic action invokes
     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
     incorrect destructor might then be invoked immediately.  In the
     case of YYERROR or YYBACKUP, subsequent parser actions might lead
     to an incorrect destructor call or verbose syntax error message
     before the lookahead is translated.  */
  YY_SYMBOL_PRINT ("-> $$ =", YY_CAST (yysymbol_kind_t, yyr1[yyn]), &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;

  *++yyvsp = yyval;

  /* Now 'shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */
  {
    const int yylhs = yyr1[yyn] - YYNTOKENS;
    const int yyi = yypgoto[yylhs] + *yyssp;
    yystate = (0 <= yyi && yyi <= YYLAST && yycheck[yyi] == *yyssp
               ? yytable[yyi]
               : yydefgoto[yylhs]);
  }

  goto yynewstate;


/*--------------------------------------.
| yyerrlab -- here on detecting error.  |
`--------------------------------------*/
yyerrlab:
  /* Make sure we have latest lookahead translation.  See comments at
     user semantic actions for why this is necessary.  */
  yytoken = yychar == YYEMPTY ? YYSYMBOL_YYEMPTY : YYTRANSLATE (yychar);
  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
      yyerror (YY_("syntax error"));
    }

  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
         error, discard it.  */

      if (yychar <= YYEOF)
        {
          /* Return failure if at end of input.  */
          if (yychar == YYEOF)
            YYABORT;
        }
      else
        {
          yydestruct ("Error: discarding",
                      yytoken, &yylval);
          yychar = YYEMPTY;
        }
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:
  /* Pacify compilers when the user code never invokes YYERROR and the
     label yyerrorlab therefore never appears in user code.  */
  if (0)
    YYERROR;

  /* Do not reclaim the symbols of the rule whose action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;      /* Each real token shifted decrements this.  */

  /* Pop stack until we find a state that shifts the error token.  */
  for (;;)
    {
      yyn = yypact[yystate];
      if (!yypact_value_is_default (yyn))
        {
          yyn += YYSYMBOL_YYerror;
          if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYSYMBOL_YYerror)
            {
              yyn = yytable[yyn];
              if (0 < yyn)
                break;
            }
        }

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
        YYABORT;


      yydestruct ("Error: popping",
                  YY_ACCESSING_SYMBOL (yystate), yyvsp);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END


  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", YY_ACCESSING_SYMBOL (yyn), yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturn;


/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturn;


#if !defined yyoverflow
/*-------------------------------------------------.
| yyexhaustedlab -- memory exhaustion comes here.  |
`-------------------------------------------------*/
yyexhaustedlab:
  yyerror (YY_("memory exhausted"));
  yyresult = 2;
  goto yyreturn;
#endif


/*-------------------------------------------------------.
| yyreturn -- parsing is finished, clean up and return.  |
`-------------------------------------------------------*/
yyreturn:
  if (yychar != YYEMPTY)
    {
      /* Make sure we have latest lookahead translation.  See comments at
         user semantic actions for why this is necessary.  */
      yytoken = YYTRANSLATE (yychar);
      yydestruct ("Cleanup: discarding lookahead",
                  yytoken, &yylval);
    }
  /* Do not reclaim the symbols of the rule whose action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
                  YY_ACCESSING_SYMBOL (+*yyssp), yyvsp);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif

  return yyresult;
}

#line 3405 "gram.y"

